package de.espirit.firstspirit.opt.example.editorialsar.clientservice;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.project.Project;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.Store;
import de.espirit.firstspirit.access.store.templatestore.FormatTemplate;
import de.espirit.firstspirit.access.store.templatestore.LinkTemplate;
import de.espirit.firstspirit.access.store.templatestore.PageTemplate;
import de.espirit.firstspirit.access.store.templatestore.SectionTemplate;
import de.espirit.firstspirit.access.store.templatestore.TableTemplate;
import de.espirit.firstspirit.access.store.templatestore.TemplateStoreRoot;
import de.espirit.firstspirit.agency.StoreAgent;
import de.espirit.firstspirit.opt.example.editorialsar.projectapp.EditorialSARProjectAppConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.projectapp.EditorialSARProjectAppStorage;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collections;
import java.util.List;
import java.util.Map;


/**
 * Client service, intended to be registered during initialization of a client session by {@link EditorialSARPermanentPlugin}.
 *
 * Carries out tasks such as obtaining search-enabled templates from the project application and persisting search settings
 * as well as a list of report items of the current search run.
 */
public class EditorialSARClientService {

	private final List<EditorialSARProjectAppConfiguration.TemplateTableObject> _pageTemplates;
	private final List<EditorialSARProjectAppConfiguration.TemplateTableObject> _sectionTemplates;
	private final List<EditorialSARProjectAppConfiguration.TemplateTableObject> _tableTemplates;
	private final List<EditorialSARProjectAppConfiguration.TemplateTableObject> _formatTemplates;
	private final List<EditorialSARProjectAppConfiguration.TemplateTableObject> _linkTemplates;

	private final BaseContext _context;


	/**
	 * Constructor. Obtain stored configuration information from {@link EditorialSARProjectAppStorage}, if available.
	 *
	 * @param projectAppStorage The {@link EditorialSARProjectAppStorage} object to use.
	 */
	EditorialSARClientService(@NotNull final EditorialSARProjectAppStorage projectAppStorage, @NotNull final BaseContext context) {
		_context = context;
		final Map<Class<?>, List<EditorialSARProjectAppConfiguration.TemplateTableObject>> templateTableObjects = projectAppStorage.load();
		if (!templateTableObjects.isEmpty()) {
			_pageTemplates = templateTableObjects.get(PageTemplate.class);
			_sectionTemplates = templateTableObjects.get(SectionTemplate.class);
			_tableTemplates = templateTableObjects.get(TableTemplate.class);
			_formatTemplates = templateTableObjects.get(FormatTemplate.class);
			_linkTemplates = templateTableObjects.get(LinkTemplate.class);
		} else {
			_pageTemplates = Collections.emptyList();
			_sectionTemplates = Collections.emptyList();
			_tableTemplates = Collections.emptyList();
			_formatTemplates = Collections.emptyList();
			_linkTemplates = Collections.emptyList();
		}
	}


	/**
	 * Determine if a given template is eligible for search and replace operations (based on project app configuration).
	 *
	 * @param template The template that should be checked for.
	 * @return {@code true} if elements based on this template may be used in search and replace, {@code false} otherwise.
	 */
	public boolean isSupportedTemplate(@Nullable final IDProvider template) {
		if (template == null) {
			return false;
		}
		final EditorialSARProjectAppConfiguration.TemplateTableObject templateTableObject = new EditorialSARProjectAppConfiguration.TemplateTableObject(template.getId(), template.getUid(), template.getDisplayName(template.getProject().getMasterLanguage()), false);
		if (template instanceof PageTemplate) {
			if (_pageTemplates.contains(templateTableObject)) {
				return _pageTemplates.get(_pageTemplates.indexOf(templateTableObject)).isEnabled();
			}
		} else if (template instanceof SectionTemplate && !(template instanceof TableTemplate)) {
			if (_sectionTemplates.contains(templateTableObject)) {
				return _sectionTemplates.get(_sectionTemplates.indexOf(templateTableObject)).isEnabled();
			}
		} else if (template instanceof TableTemplate) {
			if (_tableTemplates.contains(templateTableObject)) {
				return _tableTemplates.get(_tableTemplates.indexOf(templateTableObject)).isEnabled();
			}
		} else if (template instanceof FormatTemplate) {
			if (_formatTemplates.contains(templateTableObject)) {
				return _formatTemplates.get(_formatTemplates.indexOf(templateTableObject)).isEnabled();
			}
		} else if (template instanceof LinkTemplate) {
			if (_linkTemplates.contains(templateTableObject)) {
				return _linkTemplates.get(_linkTemplates.indexOf(templateTableObject)).isEnabled();
			}
		}
		return false;
	}


	/**
	 * Determine if the project's metadata template is eligible for search and replace operations (based on project app configuration).
	 *
	 * @return {@code true} if the metadata template may be used in search and replace, {@code false} otherwise.
	 */
	public boolean isMetadataTemplateSupported() {
		final StoreAgent storeAgent = _context.requireSpecialist(StoreAgent.TYPE);
		final Store templateStore = storeAgent.getStore(Store.Type.TEMPLATESTORE, false);
		final Project project = templateStore.getProject();
		final TemplateStoreRoot templateStoreRoot = templateStore.getProject().getUserService().getTemplateStore();
		final long metadataTemplateId = project.getMetadataTemplate();
		final PageTemplate metadataTemplate = (PageTemplate) templateStoreRoot.getTemplateById(metadataTemplateId);
		return metadataTemplate != null && isSupportedTemplate(metadataTemplate);
	}
}