package de.espirit.firstspirit.opt.example.editorialsar.projectapp;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.Store;
import de.espirit.firstspirit.access.store.templatestore.FormatTemplate;
import de.espirit.firstspirit.access.store.templatestore.LinkTemplate;
import de.espirit.firstspirit.access.store.templatestore.PageTemplate;
import de.espirit.firstspirit.access.store.templatestore.PreviewImageProvider;
import de.espirit.firstspirit.access.store.templatestore.SectionTemplate;
import de.espirit.firstspirit.access.store.templatestore.TableTemplate;
import de.espirit.firstspirit.access.store.templatestore.Template;
import de.espirit.firstspirit.access.store.templatestore.TemplateStoreElement;
import de.espirit.firstspirit.agency.SpecialistsBroker;
import de.espirit.firstspirit.agency.StoreAgent;
import de.espirit.firstspirit.agency.UIAgent;
import de.espirit.firstspirit.io.FileSystem;
import de.espirit.firstspirit.module.Configuration;
import de.espirit.firstspirit.module.ProjectEnvironment;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;


/**
 * Configuration functionality for the {@link EditorialSARProjectApp}.
 *
 * This class is used to draw a dialog that allows to enable templates for search and replace functionality (i.e., elements
 * based on these templates will be considered during search and replace operations of this module).
 */
public class EditorialSARProjectAppConfiguration implements Configuration<ProjectEnvironment> {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.editorialsar.EditorialSARMessages";
	private ResourceBundle _resourceBundle;

	private ProjectEnvironment _env;

	private EditorialSARProjectAppStorage _projectAppStorage;
	private Map<Class<?>, List<TemplateTableObject>> _templateTableObjects = new HashMap<Class<?>, List<TemplateTableObject>>();

	private static final TemplateType[] templateTypes = {
			new TemplateType(
					"Page Templates",
					PageTemplate.class
			),
			new TemplateType(
					"Section Templates",
					SectionTemplate.class
			),
			new TemplateType(
					"Table Templates",
					TableTemplate.class
			),
			new TemplateType(
					"Format Templates",
					FormatTemplate.class
			),
			new TemplateType(
					"Link Templates",
					LinkTemplate.class
			)
	};


	@findbugs.SuppressWarnings("MS_EXPOSE_REP")
	public static TemplateType[] getTemplateTypes() {
		// reference is only used internally within this sample
		return templateTypes;
	}


	@Override
	public boolean hasGui() {
		return true;
	}


	/**
	 * Create a panel with a tabbed control, one tab per template type (see {@link EditorialSARProjectAppConfiguration#templateTypes}).
	 *
	 * @param masterFrame basic frame component, for creating new windows
	 * @return A configuration panel.
	 */
	@Nullable
	@Override
	public JComponent getGui(final Frame masterFrame) {
		final SpecialistsBroker broker = _env.getBroker();

		final GridBagLayout layout = new GridBagLayout();

		final GridBagConstraints layoutConstraints = new GridBagConstraints();
		layoutConstraints.fill = GridBagConstraints.BOTH;
		layoutConstraints.insets = new Insets(5, 5, 5, 5);
		layoutConstraints.weightx = 1.0;
		layoutConstraints.weighty = 1.0;

		final JPanel panel = new JPanel(layout);
		panel.setPreferredSize(new Dimension(640, 480));
		panel.setMinimumSize(new Dimension(250, 400));

		final JTabbedPane templateTabbedPane = new JTabbedPane();
		for (final TemplateType templateType : templateTypes) {
			final TemplateTabFactory templateTabFactory = new TemplateTabFactory();
			templateTabbedPane.addTab(
					templateType.getTemplateName(),
					templateTabFactory.createTemplateTabComponent(broker, _templateTableObjects.get(templateType.getTemplateClass()))
			);
		}
		panel.add(templateTabbedPane, layoutConstraints);

		return panel;
	}


	/**
	 * Load the stored configuration data via {@link EditorialSARProjectAppStorage}.
	 */
	@Override
	public void load() {
		_templateTableObjects = _projectAppStorage.load();
	}


	/**
	 * Store the current configuration data (from the configuration panel) via {@link EditorialSARProjectAppStorage}.
	 */
	@Override
	public void store() {
		_projectAppStorage.store(_templateTableObjects);
	}


	@Override
	public Set<String> getParameterNames() {
		return Collections.emptySet();
	}


	@Nullable
	@Override
	public String getParameter(final String name) {
		return null;
	}


	/**
	 * Initialization method.
	 *
	 * Obtain the module's resource bundle and configure the various template types with localized display names.
	 * Also initialize project app storage.
	 *
	 * @param moduleName module name
	 * @param componentName component name
	 * @param env useful environment information for this component.
	 */
	@Override
	public void init(final String moduleName, final String componentName, final ProjectEnvironment env) {
		_env = env;

		final Language language = _env.getBroker().requireSpecialist(UIAgent.TYPE).getDisplayLanguage();
		_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, language.getLocale());

		for (final TemplateType templateType : templateTypes) {
			if (templateType.getTemplateClass() == PageTemplate.class) {
				templateType.setTemplateName(
						_resourceBundle.getString("configuration.template_types.page")
				);
			} else if (templateType.getTemplateClass() == SectionTemplate.class) {
				templateType.setTemplateName(
						_resourceBundle.getString("configuration.template_types.section")
				);
			} else if (templateType.getTemplateClass() == TableTemplate.class) {
				templateType.setTemplateName(
						_resourceBundle.getString("configuration.template_types.table")
				);
			} else if (templateType.getTemplateClass() == FormatTemplate.class) {
				templateType.setTemplateName(
						_resourceBundle.getString("configuration.template_types.format")
				);
			} else if (templateType.getTemplateClass() == LinkTemplate.class) {
				templateType.setTemplateName(
						_resourceBundle.getString("configuration.template_types.link")
				);
			}
		}

		final FileSystem<?> projectAppConfig = _env.getConfDir();

		_projectAppStorage = new EditorialSARProjectAppStorage(_env.getBroker(), projectAppConfig);
	}


	@Override
	public ProjectEnvironment getEnvironment() {
		return _env;
	}


	/**
	 * Utility class to store information about template types.
	 */
	public static class TemplateType {

		private String _templateName;
		private final Class<?> _templateClass;

		TemplateType(@NotNull final String templateName, @NotNull final Class<?> templateClass) {
			_templateName = templateName;
			_templateClass = templateClass;
		}

		private String getTemplateName() {
			return _templateName;
		}

		public Class<?> getTemplateClass() {
			return _templateClass;
		}

		private void setTemplateName(@NotNull final String templateName) {
			_templateName = templateName;
		}
	}


	/**
	 * Factory class to build a tab panel for a given template type, adding a table with templates of that type and
	 * space for further information such as preview image and notes.
	 */
	private class TemplateTabFactory {

		private final JTable _templateTable = new JTable();
		private final JLabel _templateNameLabel = new JLabel();
		private final JLabel _templateDescriptionLabel = new JLabel();
		private final JLabel _templatePreviewLabel = new JLabel();

		private JComponent createTemplateTabComponent(@NotNull final SpecialistsBroker broker, @NotNull final List<TemplateTableObject> templateTableObjects) {
			final StoreAgent storeAgent = broker.requireSpecialist(StoreAgent.TYPE);
			final Store templateStore = storeAgent.getStore(Store.Type.TEMPLATESTORE, false);

			final GridBagLayout layout = new GridBagLayout();

			final JPanel tabPanel = new JPanel(layout);
			tabPanel.setMinimumSize(new Dimension(250, 400));
			{
				_templateTable.setModel(new TemplateTableModel(templateTableObjects));

				final GridBagConstraints tableConstraints = new GridBagConstraints();
				tableConstraints.fill = GridBagConstraints.BOTH;
				tableConstraints.gridx = 0;
				tableConstraints.gridy = 0;
				tableConstraints.gridwidth = 2;
				tableConstraints.insets = new Insets(5, 5, 5, 5);
				tableConstraints.weightx = 1.0;
				tableConstraints.weighty = 1.0;
				tableConstraints.anchor = GridBagConstraints.FIRST_LINE_START;

				final TableColumn enabledColumn = _templateTable.getColumnModel().getColumn(0);
				enabledColumn.setMinWidth(52);
				enabledColumn.setMaxWidth(52);
				enabledColumn.setResizable(false);

				final TableColumn idColumn = _templateTable.getColumnModel().getColumn(3);
				idColumn.setMinWidth(100);
				idColumn.setMaxWidth(100);
				idColumn.setResizable(false);

				_templateTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
				_templateTable.getSelectionModel().addListSelectionListener(
						new ListSelectionListener() {
							@Override
							public void valueChanged(final ListSelectionEvent event) {
								final int viewRow = _templateTable.getSelectedRow();
								if (viewRow < 0) {
									_templateDescriptionLabel.setText("");
								} else {
									final int modelRow = _templateTable.convertRowIndexToModel(viewRow);

									_templateNameLabel.setText(
											(String) _templateTable.getModel().getValueAt(modelRow, 1)
									);

									final TemplateStoreElement template = (TemplateStoreElement) templateStore.getStoreElement(
											(Long) _templateTable.getModel().getValueAt(modelRow, 3)
									);
									if (template != null && !(template instanceof FormatTemplate)) {
										final String templateComment = ((Template) template).getComment();
										if (templateComment != null) {
											_templateDescriptionLabel.setText(templateComment);
										}
										try {
											final byte[] templatePreviewImage = ((PreviewImageProvider) template).getPreviewImage();
											if (templatePreviewImage != null && templatePreviewImage.length != 0) {
												final ImageIcon fullPreview = new ImageIcon(templatePreviewImage);

												final int widthOriginal = fullPreview.getImage().getWidth(null);
												final int heightOriginal = fullPreview.getImage().getHeight(null);
												final int widthScaled;
												final int heightScaled;
												if (widthOriginal/200.0 > heightOriginal/133.0) {
													widthScaled = 200;
													final Double scaleFactor = 200.0/widthOriginal;
													heightScaled = (int) (heightOriginal * scaleFactor);
												} else if (widthOriginal/200.0 < heightOriginal/133.0) {
													final Double scaleFactor = heightOriginal/133.0;
													widthScaled = (int) (widthOriginal / scaleFactor);
													heightScaled = 133;
												} else {
													widthScaled = 200;
													heightScaled = 133;
												}

												final BufferedImage scaledPreview = getScaledPreview(fullPreview, widthScaled, heightScaled);
												_templatePreviewLabel.setIcon(new ImageIcon(scaledPreview));
											} else {
												final BufferedImage scaledPreview = getScaledPreview(null, null, null);
												_templatePreviewLabel.setIcon(new ImageIcon(scaledPreview));
											}
										} catch (final IOException e) {
											// Nothing.
										}
									} else {
										_templateDescriptionLabel.setText("");
									}
								}
							}
				});
				_templateTable.getModel().addTableModelListener(
						new TableModelListener() {
							@Override
							public void tableChanged(final TableModelEvent e) {
								final int row = e.getFirstRow();
								final int column = e.getColumn();
								if (column == 0) {
									final TableModel model = (TableModel) e.getSource();
									final boolean enabled = (Boolean) model.getValueAt(row, column);
									final TemplateTableObject rowObject = templateTableObjects.get(row);
									rowObject.setEnabled(enabled);
									templateTableObjects.set(row, rowObject);
								}
							}
						}
				);

				tabPanel.add(new JScrollPane(_templateTable), tableConstraints);
			}
			{
				_templateNameLabel.setText("");
				_templateDescriptionLabel.setText("");

				final GridBagConstraints labelConstraints = new GridBagConstraints();
				labelConstraints.fill = GridBagConstraints.HORIZONTAL;
				labelConstraints.gridx = 1;
				labelConstraints.gridy = 1;
				labelConstraints.insets = new Insets(5, 5, 5, 5);
				labelConstraints.weightx = 1.0;
				labelConstraints.weighty = 0.0;
				labelConstraints.anchor = GridBagConstraints.LAST_LINE_START;

				_templateNameLabel.setFont(new Font(_templateNameLabel.getFont().getFamily(), Font.BOLD,  _templateNameLabel.getFont().getSize()));

				tabPanel.add(_templateNameLabel, labelConstraints);

				labelConstraints.gridy = 2;

				tabPanel.add(_templateDescriptionLabel, labelConstraints);

				labelConstraints.gridx = 0;
				labelConstraints.gridy = 1;
				labelConstraints.gridwidth = 1;
				labelConstraints.gridheight = 2;
				labelConstraints.weightx = 0.0;
				labelConstraints.weighty = 0.0;
				labelConstraints.fill = GridBagConstraints.BOTH;

				_templatePreviewLabel.setSize(new Dimension(200, 133));
				_templatePreviewLabel.setMinimumSize(new Dimension(200, 133));
				_templatePreviewLabel.setMaximumSize(new Dimension(400, 300));

				final BufferedImage scaledPreview = getScaledPreview(null, null, null);
				_templatePreviewLabel.setIcon(new ImageIcon(scaledPreview));

				tabPanel.add(_templatePreviewLabel, labelConstraints);
			}

			return tabPanel;
		}


		@NotNull
		private BufferedImage getScaledPreview(@Nullable final ImageIcon fullImage, @Nullable final Integer width, @Nullable final Integer height) {
			final int imageWidth = width != null ? width : 200;
			final int imageHeight = height != null ? height: 133;

			final BufferedImage scaledPreview = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_INT_ARGB);

			final Graphics2D graphics2D = scaledPreview.createGraphics();
			graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
			graphics2D.drawImage(fullImage != null ? fullImage.getImage() : null, 0, 0, 200, 133, null);
			graphics2D.dispose();

			return scaledPreview;
		}


		/**
		 * Table model for template listings.
		 */
		private class TemplateTableModel implements TableModel {

			private final String[] _columnNames = {
					_resourceBundle.getString("configuration.table.header.enabled"),
					_resourceBundle.getString("configuration.table.header.uid"),
					_resourceBundle.getString("configuration.table.header.template"),
					_resourceBundle.getString("configuration.table.header.store_identifier")
			};
			private final List<TemplateTableObject> _templateTableObjects;


			TemplateTableModel(@NotNull final List<TemplateTableObject> templateTableObjects) {
				_templateTableObjects = templateTableObjects;
			}


			@Override
			public int getRowCount() {
				return _templateTableObjects.size();
			}


			@Override
			public int getColumnCount() {
				return _columnNames.length;
			}


			@Override
			public String getColumnName(final int columnIndex) {
				return _columnNames[columnIndex];
			}


			@Override
			public Class<?> getColumnClass(final int columnIndex) {
				return getValueAt(0, columnIndex).getClass();
			}


			@Override
			public boolean isCellEditable(final int rowIndex, final int columnIndex) {
				return columnIndex == 0;
			}


			@Override
			public Object getValueAt(final int rowIndex, final int columnIndex) {
				switch (columnIndex) {
					case 0:
						return _templateTableObjects.get(rowIndex).isEnabled();
					case 1:
						return _templateTableObjects.get(rowIndex).getUid();
					case 2:
						return _templateTableObjects.get(rowIndex).getDisplayName();
					case 3:
						return _templateTableObjects.get(rowIndex).getID();
					default:
						return -1;
				}
			}


			@Override
			public void setValueAt(final Object aValue, final int rowIndex, final int columnIndex) {
				switch (columnIndex) {
					case 0:
						_templateTableObjects.get(rowIndex).setEnabled((Boolean) aValue);
						break;
					default:
						break;
				}
			}


			@Override
			public void addTableModelListener(final TableModelListener l) {
			}


			@Override
			public void removeTableModelListener(final TableModelListener l) {
			}
		}
	}


	/**
	 * Utility class to obtain information about template elements.
	 */
	public static final class TemplateTableObject {

		private boolean _enabled;
		private final long _id;
		private final String _uid;
		private final String _displayName;


		public TemplateTableObject(final long id, @NotNull final String uid, @NotNull final String displayName, final boolean enabled) {
			_id = id;
			_uid = uid;
			_displayName = displayName;
			_enabled = enabled;
		}


		public boolean equals(final Object templateTableObject) {
			return templateTableObject instanceof TemplateTableObject &&
				((TemplateTableObject) templateTableObject).getID() == _id &&
				((TemplateTableObject) templateTableObject).getUid().equals(_uid);
		}


		@Override
		public int hashCode() {
			int result = (int) (_id ^ (_id >>> 32));
			result = 31 * result + _uid.hashCode();
			return result;
		}


		public boolean isEnabled() {
			return _enabled;
		}


		public String getUid() {
			return _uid;
		}


		public String getDisplayName() {
			return _displayName;
		}


		public long getID() {
			return _id;
		}


		private void setEnabled(final boolean enabled) {
			_enabled = enabled;
		}
	}
}
