package de.espirit.firstspirit.opt.example.editorialsar.projectapp;

import de.espirit.common.util.CsvReader;
import de.espirit.common.util.CsvWriter;
import de.espirit.common.util.Listable;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.Store;
import de.espirit.firstspirit.access.store.templatestore.FormatTemplate;
import de.espirit.firstspirit.access.store.templatestore.LinkTemplate;
import de.espirit.firstspirit.access.store.templatestore.PageTemplate;
import de.espirit.firstspirit.access.store.templatestore.SectionTemplate;
import de.espirit.firstspirit.access.store.templatestore.TableTemplate;
import de.espirit.firstspirit.access.store.templatestore.Template;
import de.espirit.firstspirit.agency.SpecialistsBroker;
import de.espirit.firstspirit.agency.StoreAgent;
import de.espirit.firstspirit.io.FileHandle;
import de.espirit.firstspirit.io.FileSystem;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Class providing functionality to read and write project-specific configuration files for the {@link EditorialSARProjectApp}.
 */
public class EditorialSARProjectAppStorage {

	private final SpecialistsBroker _broker;
	private final FileSystem<?> _projectAppStorage;


	public EditorialSARProjectAppStorage(@NotNull final SpecialistsBroker broker, @NotNull final FileSystem<?> projectAppStorage) {
		_broker = broker;
		_projectAppStorage = projectAppStorage;
	}


	/**
	 * Load template configuration from a template class-specific configuration file, merge it with the current set of
	 * templates of that type and return these templates as a map.
	 *
	 * @return A {@link Map} of {@link Template} objects.
	 */
	public Map<Class<?>, List<EditorialSARProjectAppConfiguration.TemplateTableObject>> load() {
		final Map<Class<?>, List<EditorialSARProjectAppConfiguration.TemplateTableObject>> templateTableObjects = new HashMap<Class<?>, List<EditorialSARProjectAppConfiguration.TemplateTableObject>>();
		for (final EditorialSARProjectAppConfiguration.TemplateType templateType : EditorialSARProjectAppConfiguration.getTemplateTypes()) {
			final String fileName;
			final List<EditorialSARProjectAppConfiguration.TemplateTableObject> templatesFromConfigFile;
			final List<EditorialSARProjectAppConfiguration.TemplateTableObject> mergedList;
			if (templateType.getTemplateClass().equals(PageTemplate.class)) {
				fileName = "template_page.csv";
				templatesFromConfigFile = loadFromFile(fileName);
				mergedList = mergeWithCurrent(templatesFromConfigFile, PageTemplate.class, null);
				templateTableObjects.put(PageTemplate.class, mergedList);
			} else if (templateType.getTemplateClass().equals(SectionTemplate.class)) {
				fileName = "template_section.csv";
				templatesFromConfigFile = loadFromFile(fileName);
				mergedList = mergeWithCurrent(templatesFromConfigFile, SectionTemplate.class, TableTemplate.class);
				templateTableObjects.put(SectionTemplate.class, mergedList);
			} else if (templateType.getTemplateClass().equals(TableTemplate.class)) {
				fileName = "template_table.csv";
				templatesFromConfigFile = loadFromFile(fileName);
				mergedList = mergeWithCurrent(templatesFromConfigFile, TableTemplate.class, null);
				templateTableObjects.put(TableTemplate.class, mergedList);
			} else if (templateType.getTemplateClass().equals(FormatTemplate.class)) {
				fileName = "template_format.csv";
				templatesFromConfigFile = loadFromFile(fileName);
				mergedList = mergeWithCurrent(templatesFromConfigFile, FormatTemplate.class, null);
				templateTableObjects.put(FormatTemplate.class, mergedList);
			} else if (templateType.getTemplateClass().equals(LinkTemplate.class)) {
				fileName = "template_link.csv";
				templatesFromConfigFile = loadFromFile(fileName);
				mergedList = mergeWithCurrent(templatesFromConfigFile, LinkTemplate.class, null);
				templateTableObjects.put(LinkTemplate.class, mergedList);
			}
		}
		return templateTableObjects;
	}


	/**
	 * Load template configuration from a given file.
	 *
	 * @param fileName The file name to access.
	 * @return A {@link List} of {@link EditorialSARProjectAppConfiguration.TemplateTableObject}s.
	 */
	private List<EditorialSARProjectAppConfiguration.TemplateTableObject> loadFromFile(@NotNull final String fileName) {
		if (!fileName.isEmpty()) {
			try {
				final FileHandle fileHandle = _projectAppStorage.obtain(fileName);
				if (fileHandle.exists()) {
					final List<EditorialSARProjectAppConfiguration.TemplateTableObject> templateTableObjects = new ArrayList<EditorialSARProjectAppConfiguration.TemplateTableObject>();
					final CsvReader csvReader = new CsvReader(new InputStreamReader(fileHandle.load()));
					CsvReader.CsvToken csvToken = csvReader.read();
					while (csvToken.getType() == CsvReader.CsvToken.NEW_LINE) {
						csvToken = csvReader.read();
					}
					while (csvToken.getType() != CsvReader.CsvToken.EOF) {
						Long id = null;
						String uid = null;
						String displayName = null;
						Boolean enabled = null;
						if (csvToken.getType() != CsvReader.CsvToken.NEW_LINE) {
							id = Long.valueOf(csvToken.getValue());
							csvToken = csvReader.read();
						}
						if (csvToken.getType() != CsvReader.CsvToken.NEW_LINE) {
							uid = csvToken.getValue();
							csvToken = csvReader.read();
						}
						if (csvToken.getType() != CsvReader.CsvToken.NEW_LINE) {
							displayName = csvToken.getValue();
							csvToken = csvReader.read();
						}
						if (csvToken.getType() != CsvReader.CsvToken.NEW_LINE) {
							enabled = Boolean.valueOf(csvToken.getValue());
							csvToken = csvReader.read();
						}
						if (csvToken.getType() == CsvReader.CsvToken.NEW_LINE && id != null && uid != null && displayName != null && enabled != null) {
							final EditorialSARProjectAppConfiguration.TemplateTableObject templateTableObject = new EditorialSARProjectAppConfiguration.TemplateTableObject(id, uid, displayName, enabled);
							templateTableObjects.add(templateTableObject);
							csvToken = csvReader.read();
						} else {
							throw new IllegalStateException("Malformed comma-separated value (CSV) file. Cannot read template configuration.");
						}
					}
					return templateTableObjects;
				}
			} catch (final IOException e) {
				throw new IllegalStateException("An error occurred during loadFromFile.", e);
			}
		}
		return Collections.emptyList();
	}


	/**
	 * Merge a list of templates as read from a configuration file with the list of templates currently available in the project
	 * so that templates not present in the configuration file are properly represented in the project app's configuration panel.
	 *
	 * @param templatesFromConfigFile The {@link List} of {@link EditorialSARProjectAppConfiguration.TemplateTableObject}s sourced from a configuration file.
	 * @param includeClass The template types to include in the merge process.
	 * @param ignoreClass The template types to exclude in the merge process.
	 * @return A {@link List} of {@link EditorialSARProjectAppConfiguration.TemplateTableObject}s.
	 */
	private List<EditorialSARProjectAppConfiguration.TemplateTableObject> mergeWithCurrent(@NotNull final List<EditorialSARProjectAppConfiguration.TemplateTableObject> templatesFromConfigFile, @NotNull final Class<? extends IDProvider> includeClass, @Nullable final Class<? extends IDProvider> ignoreClass) {
		final List<EditorialSARProjectAppConfiguration.TemplateTableObject> mergedList = new ArrayList<EditorialSARProjectAppConfiguration.TemplateTableObject>();
		final StoreAgent storeAgent = _broker.requireSpecialist(StoreAgent.TYPE);
		final Store templateStore = storeAgent.getStore(Store.Type.TEMPLATESTORE, false);
		final Language masterLanguage = templateStore.getProject().getMasterLanguage();
		final Listable<? extends IDProvider> templates = templateStore.getChildren(includeClass, true);
		for (final IDProvider template : templates) {
			// If a not-null ignoreClass parameter is provided, determine if the template element implements this ignored class.
			boolean templateIgnored = false;
			if (ignoreClass != null) {
				final List<Class<?>> templateInterfaces = Arrays.asList(template.getClass().getInterfaces());
				templateIgnored = templateInterfaces.contains(ignoreClass);
			}

			// If the template is not ignored (based on the ignoreClass parameter), add it to the merged list of template table objects.
			if (!templateIgnored) {
				final EditorialSARProjectAppConfiguration.TemplateTableObject templateTableObject = new EditorialSARProjectAppConfiguration.TemplateTableObject(template.getId(), template.getUid(), template.getDisplayName(masterLanguage), false);
				if (!templatesFromConfigFile.contains(templateTableObject)) {
					mergedList.add(templateTableObject);
				} else {
					mergedList.add(templatesFromConfigFile.get(templatesFromConfigFile.indexOf(templateTableObject)));
				}
			}
		}
		return mergedList;
	}


	/**
	 * Write template-specific data to a template type-specific configuration file.
	 *
	 * @param templateTableObjects A {@link Map} of {@link EditorialSARProjectAppConfiguration.TemplateTableObject}s containing configuration information.
	 */
	public void store(@NotNull final Map<Class<?>, List<EditorialSARProjectAppConfiguration.TemplateTableObject>> templateTableObjects) {
		for (final EditorialSARProjectAppConfiguration.TemplateType templateType : EditorialSARProjectAppConfiguration.getTemplateTypes()) {
			final String fileName;
			if (templateType.getTemplateClass().equals(PageTemplate.class)) {
				fileName = "template_page.csv";
				storeToFile(fileName, templateTableObjects.get(PageTemplate.class));
			} else if (templateType.getTemplateClass().equals(SectionTemplate.class)) {
				fileName = "template_section.csv";
				storeToFile(fileName, templateTableObjects.get(SectionTemplate.class));
			} else if (templateType.getTemplateClass().equals(TableTemplate.class)) {
				fileName = "template_table.csv";
				storeToFile(fileName, templateTableObjects.get(TableTemplate.class));
			} else if (templateType.getTemplateClass().equals(FormatTemplate.class)) {
				fileName = "template_format.csv";
				storeToFile(fileName, templateTableObjects.get(FormatTemplate.class));
			} else if (templateType.getTemplateClass().equals(LinkTemplate.class)) {
				fileName = "template_link.csv";
				storeToFile(fileName, templateTableObjects.get(LinkTemplate.class));
			}
		}
	}


	/**
	 * Write template-specific data to a given configuration file.
	 *
	 * @param fileName The file name to access.
	 * @param templateTableObjects A {@link List} of {@link EditorialSARProjectAppConfiguration.TemplateTableObject}s containing configuration information.
	 */
	private void storeToFile(@NotNull final String fileName, @NotNull final List<EditorialSARProjectAppConfiguration.TemplateTableObject> templateTableObjects) {
		try {
			final FileHandle fileHandle = _projectAppStorage.obtain(fileName);
			final CsvWriter csvWriter = new CsvWriter(new OutputStreamWriter(fileHandle.getOutputStream()));
			for (final EditorialSARProjectAppConfiguration.TemplateTableObject templateTableObject : templateTableObjects) {
				csvWriter.write(((Long) templateTableObject.getID()).toString());
				csvWriter.write(templateTableObject.getUid());
				csvWriter.write(templateTableObject.getDisplayName());
				csvWriter.write(((Boolean) templateTableObject.isEnabled()).toString());
				csvWriter.writeNewLine();
			}
			csvWriter.close();
		} catch (final IOException e) {
			throw new IllegalStateException("An error occurred during storeToFile.", e);
		}
	}
}
