package de.espirit.firstspirit.opt.example.editorialsar.report;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.ServicesBroker;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.Store;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.access.store.mediastore.Media;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.access.store.templatestore.TemplateStoreElement;
import de.espirit.firstspirit.agency.MultiFormValidationReport;
import de.espirit.firstspirit.agency.QueryAgent;
import de.espirit.firstspirit.agency.SpecialistsBroker;
import de.espirit.firstspirit.agency.StoreAgent;
import de.espirit.firstspirit.agency.ValidationAgent;
import de.espirit.firstspirit.client.plugin.report.DataProvider;
import de.espirit.firstspirit.client.plugin.report.Parameter;
import de.espirit.firstspirit.client.plugin.report.ParameterBoolean;
import de.espirit.firstspirit.client.plugin.report.ParameterMap;
import de.espirit.firstspirit.client.plugin.report.ParameterSelect;
import de.espirit.firstspirit.client.plugin.report.ParameterText;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.DatasetReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.MediaReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.PageReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.ReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.SectionReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.TemplateReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.UUID;


/**
 * Provides a set of report entry objects (of type {@link ReportEntry}) to be displayed in a report.
 */
public class EditorialSARDataProvider implements DataProvider<ReportEntry> {

	private final BaseContext _context;
	private final EditorialSARClientService _clientService;

	private final ParameterText _searchWordsParameter;
	private final ParameterText _replacementWordsParameter;
	private final ParameterSelect _searchScopeParameter;
	private final ParameterSelect _searchLanguageAbbreviationParameter;
	private final ParameterBoolean _caseSensitiveParameter;

	private SearchConfiguration _searchConfiguration;
	private Iterator<IDProvider> _queryAgentAnswerIterator;

	private final List<Page> _pagesEncountered = new ArrayList<Page>();
	private final List<ReportEntry> _filteredSearchResults = new ArrayList<ReportEntry>();

	private boolean _running;


	/**
	 * Constructor.
	 *
	 * @param context A context object used to access project and system functionality.
	 * @param parameters A map of the parameters used in the associated report plug-in.
	 */
	EditorialSARDataProvider(@NotNull final BaseContext context, @NotNull final Map<String, Parameter<?>> parameters) {
		_context = context;

		// Obtain the Editorial SAR client service.
		_clientService = _context.requireSpecialist(ServicesBroker.TYPE).getService(EditorialSARClientService.class);

		// Obtain the report's parameterization.
		_searchWordsParameter = (ParameterText) parameters.get("search_words");
		_replacementWordsParameter = (ParameterText) parameters.get("replacement_words");
		_searchScopeParameter = (ParameterSelect) parameters.get("search_scope");
		_searchLanguageAbbreviationParameter = (ParameterSelect) parameters.get("search_languages_abbreviation");
		_caseSensitiveParameter = (ParameterBoolean) parameters.get("case_sensitive");
	}


	/**
	 * Gather a list of all report entry objects that fit the report's parameters.
	 *
	 * These report entries are not just yet provided for display in the report - that happens incrementally, as the
	 * method {@link #getNext(int)} is called.
	 *
	 * @param broker A specialists broker.
	 * @param parameter Report parameter values.
	 */
	@Override
	public void start(final SpecialistsBroker broker, final ParameterMap parameter) {
		// Obtain the list of languages to conduct a search in.
		final String searchWords = parameter.get(_searchWordsParameter);
		final String replacementWords = parameter.get(_replacementWordsParameter);
		final String searchScopeString = parameter.get(_searchScopeParameter);
		final SearchConfiguration.Scope searchScope = SearchConfiguration.Scope.valueOf(searchScopeString);
		final List<Language> searchLanguages = getSearchLanguages(parameter.get(_searchLanguageAbbreviationParameter));
		final Boolean caseSensitive = parameter.get(_caseSensitiveParameter);

		// If no languages are defined, search words are not provided, or the search scope parameter didn't contain a valid scope, don't attempt a search at all.
		if (searchLanguages.isEmpty() || searchWords == null || searchWords.isEmpty() || searchScope == null) {
			return;
		}

		// Create a search configuration from the provided report parameterization.
		_searchConfiguration = new SearchConfiguration(searchLanguages, searchWords, replacementWords, searchScope, caseSensitive != null ? caseSensitive : false);

		// If the search configuration is valid for conducting at least a search run...
		if (_searchConfiguration.isValidForSearch()) {
			// Create a store-specific search query.
			final QueryAgent queryAgent = broker.requireSpecialist(QueryAgent.TYPE);
			final StringBuilder query = new StringBuilder();
			query.append(_searchConfiguration.getSearchWords()).append(" and fs.workflowLock = - and ");
			if (searchScope == SearchConfiguration.Scope.WEB_PAGES || searchScope == SearchConfiguration.Scope.PAGES_AND_SECTIONS) {
				query.append("(fs.type = Page or fs.type = Section)");
			} else if (searchScope == SearchConfiguration.Scope.DATASETS) {
				query.append("fs.type = Dataset");
			} else if (searchScope == SearchConfiguration.Scope.MEDIA) {
				query.append("fs.type = Media");
			} else if (searchScope == SearchConfiguration.Scope.TEMPLATES) {
				query.append("(fs.type = PageTemplate or fs.type = SectionTemplate or fs.type = FormatTemplate or fs.type = LinkTemplate or fs.type = TableTemplate)");
			}

			// Get a list of store elements that match the search query.
			final Iterable<IDProvider> queryAgentAnswer = queryAgent.answer(query.toString());
			_queryAgentAnswerIterator = queryAgentAnswer.iterator();
			_running = true;
		}
	}


	/**
	 * Provide a subset of report entries gathered in {@link #start(SpecialistsBroker, ParameterMap)}.
	 *
	 * To prevent returning duplicates, this method removes the report entries returned here from the overall list of
	 * report entries.
	 *
	 * @param count The requested (maximum) count of report entries to return.
	 * @return A {@link List} of report entries to be displayed in the report. This list must contain at most {@code count} number of objects.
	 */
	@Override
	public List<ReportEntry> getNext(final int count) {
		if (_queryAgentAnswerIterator != null) {
			final List<ReportEntry> results = new ArrayList<ReportEntry>();
			while (_queryAgentAnswerIterator.hasNext() && results.size() < count) {
				final ReportEntry reportEntry = getReportEntry(_queryAgentAnswerIterator.next());
				if (reportEntry != null) {
					_filteredSearchResults.add(reportEntry);
					results.add(reportEntry);
				}
			}

			if (!_queryAgentAnswerIterator.hasNext()) {
				_running = false;
			}
			return results;
		}
		return Collections.emptyList();
	}


	/**
	 * Utility method to process the next store element available in the QueryAgent's answer and generate a {@link ReportEntry} object based on
	 * this element if the element contains data processable by Editorial Search and Replace.
	 *
	 * @return A {@link ReportEntry} object or {@code null} if the element contains no data processable by this module.
	 */
	@Nullable
	private ReportEntry getReportEntry(@NotNull final IDProvider element) {
		// Process the found elements, creating a report entry object for each.

		try {
			// Exclude an element which is locked (in whichever session).
			if (element.isLockedOnServer(true)) {
				return null;
			}

			// Exclude an element on which the current user does not have at least the permission CAN_CHANGE.
			// (For Section elements, the method Permission#canChange() considers the grandparent Page element.)
			if (!element.getPermission().canChange()) {
				return null;
			}

			// Exclude an element that does not meet ON_SAVE validation requirements.
			final ValidationAgent validationAgent = _context.requireSpecialist(ValidationAgent.TYPE);
			final MultiFormValidationReport multiFormValidationReport = validationAgent.validate(element, ValidationAgent.ValidationScope.SAVE);
			if (!multiFormValidationReport.isValid()) {
				return null;
			}

			final SearchConfiguration.Scope searchScope = _searchConfiguration.getSearchScope();

			// Process page store elements.
			if (searchScope == SearchConfiguration.Scope.WEB_PAGES || searchScope == SearchConfiguration.Scope.PAGES_AND_SECTIONS) {
				if (element instanceof Page) {
					final PageReportEntry resultEntry = new PageReportEntry(_clientService, _searchConfiguration, (Page) element);
					if (searchScope == SearchConfiguration.Scope.WEB_PAGES) {
						resultEntry.findAndAddSections();
					}
					resultEntry.doSearch();
					if (resultEntry.getResultCount() > 0) {
						_pagesEncountered.add((Page) element);
						if (_context.is(BaseContext.Env.WEBEDIT)) {
							// Only consider this report entry a valid search result if a page reference references the page (ContentCreator compatibility).
							if (resultEntry.getWebeditTargetElement() != null) {
								return resultEntry;
							}
						} else {
							return resultEntry;
						}
					}
				} else if (element instanceof Section) {
					if (!element.isDeleted()) {
						if (searchScope == SearchConfiguration.Scope.WEB_PAGES) {
							// Page-based search (for ContentCreator): view page and child sections as one unit, create only one report entry per page.
							final IDProvider parent = element.getParent();
							if (parent != null) {
								final IDProvider grandParent = parent.getParent();
								if (grandParent != null) {
									if (!_pagesEncountered.contains(grandParent)) {
										final PageReportEntry resultEntry = new PageReportEntry(_clientService, _searchConfiguration, (Page) grandParent);
										resultEntry.findAndAddSections();
										resultEntry.doSearch();
										// Only consider this report entry a valid search result if a page reference references the page (ContentCreator compatibility).
										if (resultEntry.getResultCount() > 0 && resultEntry.getWebeditTargetElement() != null) {
											_pagesEncountered.add((Page) grandParent);
											return resultEntry;
										}
									}
								}
							}
						} else {
							// Page and section-based search (for SiteArchitect): create individual report entries for each page and section.
							final SectionReportEntry sectionResultEntry = new SectionReportEntry(_clientService, _searchConfiguration, (Section<?>) element);
							sectionResultEntry.doSearch();
							if (sectionResultEntry.getResultCount() > 0) {
								return sectionResultEntry;
							}
						}
					}
				}

			// Process content store elements.
			} else if (searchScope == SearchConfiguration.Scope.DATASETS) {
				if (element instanceof Dataset) {
					final DatasetReportEntry resultEntry = new DatasetReportEntry(_clientService, _searchConfiguration, (Dataset) element);
					resultEntry.doSearch();
					if (resultEntry.getResultCount() > 0) {
						return resultEntry;
					}
				}

			// Process media store elements.
			} else if (searchScope == SearchConfiguration.Scope.MEDIA) {
				if (element instanceof Media) {
					final MediaReportEntry resultEntry = new MediaReportEntry(_clientService, _searchConfiguration, (Media) element);
					resultEntry.doSearch();
					if (resultEntry.getResultCount() > 0) {
						return resultEntry;
					}
				}

			// Process template store elements.
			} else if (searchScope == SearchConfiguration.Scope.TEMPLATES) {
				if (element instanceof TemplateStoreElement) {
					final TemplateReportEntry resultEntry = new TemplateReportEntry(_clientService, _searchConfiguration, (TemplateStoreElement) element);
					resultEntry.doSearch();
					if (resultEntry.getResultCount() > 0) {
						return resultEntry;
					}
				}
			}
		} catch (final NoSuchElementException e) {
			return null;
		}

		return null;
	}


	/**
	 * Return the provider's running status (i.e., the provider is still gathering the overall list of report entries
	 * in {@link #start(SpecialistsBroker, ParameterMap)}).
	 *
	 * @return A boolean that indicates the provider's running status.
	 */
	@Override
	public boolean isRunning() {
		return _running;
	}


	/**
	 * Return the total amount of report entries found during the last {@link #start(SpecialistsBroker, ParameterMap)} run.
	 *
	 * @return The amount of report entries found.
	 */
	@Override
	public int getTotal() {
		return _filteredSearchResults.size();
	}


	/**
	 * Perform clean-up.
	 */
	@Override
	public void close() {
	}


	/**
	 * Provide a {@link String} representation of a report entry's {@link UUID} for serialization purposes.
	 *
	 * @param object The report entry to serialize.
	 * @return A {@link String} representation of a report entry's {@link UUID}.
	 */
	@Nullable
	@Override
	public String serialize(@NotNull final ReportEntry object) {
		return String.valueOf(_filteredSearchResults.indexOf(object));
	}


	/**
	 * Provide a report entry matching the {@link UUID} indicated by the serialization {@link String}.
	 *
	 * @param string A serialization {@link String}, previously returned by {@link #serialize(Object)}.
	 * @return The {@link ReportEntry} object matching the {@link UUID} contained in {@code string}.
	 */
	@Nullable
	@Override
	public ReportEntry deserialize(@NotNull final String string) {
		return _filteredSearchResults.get(Integer.valueOf(string));
	}


	/**
	 * Convert the plain-text report filter value for project language to a list of languages.
	 * @param searchLanguageAbbreviation The language abbreviation of the current report filter setting.
	 * @return A list of project languages matching this language abbreviation.
	 */
	@NotNull
	private List<Language> getSearchLanguages(@Nullable final String searchLanguageAbbreviation) {
		if (searchLanguageAbbreviation != null && !searchLanguageAbbreviation.isEmpty()) {
			// Obtain a list of all project languages.
			final StoreAgent storeAgent = _context.requireSpecialist(StoreAgent.TYPE);
			final Store pageStore = storeAgent.getStore(Store.Type.PAGESTORE, false);
			final List<Language> projectLanguages = pageStore.getProject().getLanguages();

			if ("ALL".equals(searchLanguageAbbreviation)) {
				// If the report's filter setting is set to process all languages, simply return the list of project languages.
				return projectLanguages;
			} else {
				// If the report's filter setting is set to process a single language, match this language to the project language list and return only that language in a Collection.
				for (final Language projectLanguage : projectLanguages) {
					if (projectLanguage.getAbbreviation().toLowerCase().equals(searchLanguageAbbreviation.toLowerCase())) {
						return Collections.singletonList(projectLanguage);
					}
				}
			}
		}

		// The report's language filter was set to a language that doesn't match any of the project languages.
		throw new IllegalStateException("Language filter has a null value or was configured with a non-project language.");
	}

}