package de.espirit.firstspirit.opt.example.editorialsar.report;

import de.espirit.common.imaging.ImageUtil;
import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.EditorValue;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.access.store.mediastore.File;
import de.espirit.firstspirit.access.store.mediastore.Media;
import de.espirit.firstspirit.access.store.mediastore.Picture;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.access.store.templatestore.PreviewImageProvider;
import de.espirit.firstspirit.agency.SnippetAgent;
import de.espirit.firstspirit.agency.UIAgent;
import de.espirit.firstspirit.client.plugin.report.DataRenderer;
import de.espirit.firstspirit.client.plugin.report.Image;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.ReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.store.snippets.SnippetProvider;
import de.espirit.firstspirit.store.access.templatestore.SnippetSupport;
import de.espirit.firstspirit.webedit.WebeditUiAgent;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import java.io.IOException;
import java.util.List;
import java.util.ResourceBundle;


/**
 * Class used to provide rendering information for {@link ReportEntry} objects.
 *
 * For further documentation, see the interface {@link DataRenderer}.
 */
public class EditorialSARDataRenderer implements DataRenderer<ReportEntry> {

	private final BaseContext _context;
	private final ResourceBundle _resourceBundle;


	EditorialSARDataRenderer(@NotNull final BaseContext context, @NotNull final ResourceBundle resourceBundle) {
		_context = context;
		_resourceBundle = resourceBundle;
	}


	@Nullable
	@Override
	public Image<?> getIcon(final ReportEntry object) {
		if (_context.is(BaseContext.Env.WEBEDIT)) {
			// Obtain a ContentCreator-compatible target element and use element-specific image URLs to create a report entry icon.
			final IDProvider storeElement = object.getWebeditTargetElement();
			if (storeElement instanceof Page || storeElement instanceof PageRef) {
				return Image.Factory.fromUrl("media/store/pageref.png");
			} else if (storeElement instanceof Dataset) {
				return Image.Factory.fromUrl("media/store/dataset.png");
			} else if (storeElement instanceof Media) {
				if (((Media) storeElement).getType() == Media.PICTURE) {
					return Image.Factory.fromUrl("media/store/media-picture.png");
				} else if (((Media) storeElement).getType() == Media.FILE) {
					return Image.Factory.fromUrl("media/store/media-file.png");
				}
			}
		} else {
			// Use the SiteArchitect-specific UIAgent to create a report entry icon based on element type.
			final UIAgent uiAgent = _context.requestSpecialist(UIAgent.TYPE);
			if (uiAgent != null) {
				final ImageIcon elementIcon = uiAgent.getIcon(object.getElement());
				return Image.Factory.fromIcon(elementIcon);
			}
		}
		return null;
	}


	@Nullable
	@Override
	public Image<?> getThumbnail(final ReportEntry object) {
		if (object.getElement() instanceof PreviewImageProvider) {
			// Obtain a preview image from a template store element, if available, and return an Image based upon it.
			try {
				final byte[] previewImage = ((PreviewImageProvider) object.getElement()).getPreviewImage();
				if (previewImage != null && previewImage.length > 0) {
					final ImageIcon previewImageIcon = new ImageIcon(previewImage);
					return Image.Factory.fromIcon(previewImageIcon);
				}
			} catch (final IOException e) {
				return null;
			}
		} else {
			// Obtain the best language (depending on search settings: all languages or a single language) to look for thumbnail media in.
			final Language language = getBestLanguage(object);

			// Obtain an Image from the report entry object.
			final de.espirit.firstspirit.agency.Image<?> image = getThumbnail(object, language);

			if (image != null) {
				return image.supply(new de.espirit.firstspirit.agency.Image.Consumer<Image<?>>() {
					@NotNull
					@Override
					public Image<?> consume(@NotNull Icon icon) {
						return Image.Factory.fromIcon(icon);
					}
					@Nullable
					@Override
					public Image<?> consume(@NotNull Media media) {
						return getThumbnailFromMedia(media, language);
					}
					@Nullable
					@Override
					public Image<?> consume(@NotNull String url) {
						return Image.Factory.fromUrl(url);
					}
				});
			}
		}

		// If no suitable thumbnail was found, don't display one.
		return null;
	}


	/**
	 * Return a best-match {@link Media} element to use as a thumbnail for the provided report entry and language.
	 *
	 * @param object The report entry to identify a thumbnail medium for.
	 * @param language The language to inspect the report entry in.
	 * @return A {@link Media} element.
	 */
	@Nullable
	private de.espirit.firstspirit.agency.Image<?> getThumbnail(@NotNull final ReportEntry object, @NotNull final Language language) {
		final SnippetAgent snippetAgent = _context.requireSpecialist(SnippetAgent.TYPE);
		IDProvider element = object.getElement();
		if (_context.is(BaseContext.Env.WEBEDIT) && element instanceof Page) {
			// If the report entry's element is a Page element, obtain a compatible PageRef element and obtain a
			// thumbnail from that PageRef (may refer to a different Page).
			final IDProvider targetElement = object.getWebeditTargetElement();
			if (targetElement instanceof PageRef) {
				element = targetElement;
			}
		}
		final SnippetProvider snippetProvider = snippetAgent.getSnippetProvider(element);
		return snippetProvider.getThumbnail(language);
	}


	/**
	 * Provide an {@link Image} object based upon a {@link Media} element, considering language-dependent media.
	 *
	 * @param element The {@link Media} element to create an {@link Image} object for.
	 * @param language The project {@link Language} to consider.
	 * @return An {@link Image} object.
	 */
	@Nullable
	private Image<?> getThumbnailFromMedia(@NotNull final Media element, @NotNull final Language language) {
		if (_context.is(BaseContext.Env.WEBEDIT)) {
			// Obtain and return a URL-based Image based on the provided thumbnail medium for use in ContentCreator.
			return Image.Factory.fromUrl(getThumbnailUrl(element, language, 58, 29));
		} else {
			// Obtain and return a SiteArchitect-compatible Image based on Media type (Picture or File).
			if (element.getType() == Media.PICTURE) {
				final Picture thumbnailPicture = element.getPicture(language);
				if (thumbnailPicture != null) {
					try {
						return Image.Factory.fromIcon(new ImageIcon(thumbnailPicture.getPreviewImage()));
					} catch (final IOException e) {
						return null;
					}
				}
			} else if (element.getType() == Media.FILE) {
				final File thumbnailFile = element.getFile(language);
				if (thumbnailFile != null) {
					try {
						final byte[] previewImage = thumbnailFile.getPreviewImage();
						if (previewImage != null) {
							return Image.Factory.fromIcon(new ImageIcon(previewImage));
						}
					} catch (final IOException e) {
						return null;
					}
				}
			}
		}
		return null;
	}


	/**
	 * Generate a relative URL for the given {@link Media} element, {@link Language}, and dimensions.
	 *
	 * @param element The {@link Media} element.
	 * @param language The {@link Language}.
	 * @param width Maximum width of the image.
	 * @param height Maximum height of the image.
	 * @return A relative URL.
	 */
	@NotNull
	private static String getThumbnailUrl(@NotNull final Media element, @NotNull final Language language, final int width, final int height) {
		return "PreviewImage?" +
				"store=" + element.getStore().getType().name().toLowerCase() +
				"&id=" + element.getLongID().toString() +
				"&width=" + width +
				"&height=" + height +
				"&language=" + language.getAbbreviation().toUpperCase();
	}


	@NotNull
	@Override
	public String getTitle(final ReportEntry object) {
		// Obtain the best language (depending on search settings: all languages or a single language) to look for title information in.
		final Language language = getBestLanguage(object);

		// SnippetSupport and SnippetProvider are non-API interfaces. In the future, similar functionality
		// may be made available in the FirstSpirit Access or Developer APIs (interface names, packages,
		// methods etc. are subject to change).
		if (object.getElement() instanceof SnippetSupport) {
			final SnippetProvider elementSnippetProvider = ((SnippetSupport) object.getElement()).getSnippetProvider(_context);
			if (elementSnippetProvider != null) {
				final String elementHeader = elementSnippetProvider.getHeader(language);
				if (!elementHeader.isEmpty()) {
					return elementHeader;
				}
			}
		}

		// Fallback: return the element's display name in the language identified above.
		return object.getElement().getDisplayName(language);
	}


	@Nullable
	@Override
	public String getDescription(final ReportEntry object) {
		final StringBuilder description = new StringBuilder();
		description.append(object.getHitCount())
				.append(' ');
		if (object.getHitCount() > 1) {
			description.append(_resourceBundle.getString("report.entry.details.hit.plural"));
		} else {
			description.append(_resourceBundle.getString("report.entry.details.hit.singular"));
		}
		description.append(' ')
				.append(_resourceBundle.getString("report.entry.details.in"))
				.append(' ')
				.append(object.getResultCount())
				.append(' ');
		if (object.getResultCount() > 1) {
			description.append(_resourceBundle.getString("report.entry.details.editor.plural"));
		} else {
			description.append(_resourceBundle.getString("report.entry.details.editor.singular"));
		}
		description.append(" (");
		if (object.getResultLanguages().size() == 1 && object.getResultLanguages().get(0).equals(EditorValue.SOLE_LANGUAGE)) {
			description.append(_resourceBundle.getString("report.entry.details.languageindependent"));
		} else {
			description.append(object.getResultLanguages().size())
					.append(' ');
			if (object.getResultLanguages().size() > 1) {
				description.append(_resourceBundle.getString("report.entry.details.language.plural"));
			} else {
				description.append(_resourceBundle.getString("report.entry.details.language.singular"));
			}
		}
		description.append(')');

		return description.toString();
	}


	@Nullable
	@Override
	public String getDetails(final ReportEntry object) {
		final IDProvider element = object.getWebeditTargetElement();
		if (element != null) {
			final Language language = getBestLanguage(object);
			final StringBuilder html = new StringBuilder();
			html.append("<span style=\"display: inline-block; min-width: 300px; max-width: 450px;\">");
			final de.espirit.firstspirit.agency.Image<?> thumbnail = getThumbnail(object, language);
			if (thumbnail != null) {
				final String url = thumbnail.supply(new de.espirit.firstspirit.agency.Image.Consumer<String>() {
					@Nullable
					@Override
					public String consume(@NotNull Icon icon) {
						return ImageUtil.computeFittingDataUrl(ImageUtil.iconToImage(icon), 100, 50);
					}
					@Nullable
					@Override
					public String consume(@NotNull Media media) {
						return getThumbnailUrl(media, language, 100, 50);
					}
					@Nullable
					@Override
					public String consume(@NotNull String url) {
						return url;
					}
				});
				if (url != null) {
					html.append("<img src=\"").append(url).append("\" style=\"float: left; margin-right: 15px; vertical-align: bottom;\"/>");
				}
			}
			html.append("<h3 style=\"margin-top: 0; margin-bottom: 0; clear; right;\">");
			final Image<?> elementIcon = getIcon(object);
			if (elementIcon != null) {
				html.append("<img src=\"")
						.append((String) elementIcon.get())
						.append("\" style\"vertical-align: text-center; margin-right: 5px;\"/>");
			}
			html.append(element.getDisplayName(language))
					.append("</h3>")
					.append("</span>");

			return html.toString();
		}

		// Fallback: display "no further information" string.
		return _resourceBundle.getString("report.entry.flyout.no_further_information");
	}


	/**
	 * Identify the best-match language to use to render report entry information.
	 *
	 * If search settings indicate that a single specific project language should be searched, return that language.
	 * Otherwise, determine and return the client user interface language.
	 *
	 * @return A {@link Language}.
	 */
	@NotNull
	private Language getBestLanguage(@NotNull final ReportEntry object) {
		// Obtain the list of languages to be searched (as stored in the SearchConfiguration object).
		final SearchConfiguration searchConfiguration = object.getSearchConfiguration();
		final List<Language> languages = searchConfiguration.getLanguages();

		if (languages.size() == 1 && languages.get(0) != EditorValue.SOLE_LANGUAGE) {
			// The search filter indicates a specific language should be searched. Use that language.
			return languages.get(0);
		} else if (languages.size() > 1) {
			// The search filter indicates several languages should be searched. In this case, use the client's user interface language.
			if (_context.is(BaseContext.Env.WEBEDIT)) {
				final WebeditUiAgent webeditUiAgent = _context.requireSpecialist(WebeditUiAgent.TYPE);
				return webeditUiAgent.getDisplayLanguage();
			} else {
				final UIAgent uiAgent = _context.requireSpecialist(UIAgent.TYPE);
				return uiAgent.getDisplayLanguage();
			}
		}

		// Failed to identify a good display language. This indicates erroneous search settings.
		throw new IllegalStateException("Trying to render a report snippet with invalid project language information.");
	}
}