package de.espirit.firstspirit.opt.example.editorialsar.report;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.ServiceNotFoundException;
import de.espirit.firstspirit.access.ServicesBroker;
import de.espirit.firstspirit.access.project.Project;
import de.espirit.firstspirit.access.store.Store;
import de.espirit.firstspirit.agency.StoreAgent;
import de.espirit.firstspirit.agency.UIAgent;
import de.espirit.firstspirit.client.plugin.ReportPlugin;
import de.espirit.firstspirit.client.plugin.report.DataProvider;
import de.espirit.firstspirit.client.plugin.report.DataRenderer;
import de.espirit.firstspirit.client.plugin.report.Image;
import de.espirit.firstspirit.client.plugin.report.Parameter;
import de.espirit.firstspirit.client.plugin.report.ParameterBoolean;
import de.espirit.firstspirit.client.plugin.report.ParameterSelect;
import de.espirit.firstspirit.client.plugin.report.ParameterText;
import de.espirit.firstspirit.client.plugin.report.ReportItem;
import de.espirit.firstspirit.client.plugin.report.TransferHandler;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARPermanentPlugin;
import de.espirit.firstspirit.opt.example.editorialsar.projectapp.EditorialSARProjectApp;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.ReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.item.NavigateToElementJavaClientExecutableReportItem;
import de.espirit.firstspirit.opt.example.editorialsar.report.item.NavigateToElementWebeditExecutableReportItem;
import de.espirit.firstspirit.opt.example.editorialsar.report.item.ReplaceAllJavaClientExecutableReportItem;
import de.espirit.firstspirit.opt.example.editorialsar.report.item.ReplaceAllWebeditExecutableReportItem;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.webedit.WebeditUiAgent;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.ImageIcon;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;


/**
 * Editorial Search and Replace report implementation which uses objects of type {@link ReportEntry} to represent report entries.
 *
 * For documentation on most methods, see the interface {@link ReportPlugin}.
 */
public class EditorialSARReportPlugin implements ReportPlugin<ReportEntry> {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.editorialsar.EditorialSARMessages";
	private ResourceBundle _resourceBundle;
	private Language _displayLanguage;

	private BaseContext _context;

	private ParameterText _searchWords;
	private ParameterText _replacementWords;
	private ParameterSelect _searchScope;
	private ParameterBoolean _caseSensitive;
	private ParameterSelect _searchLanguages;


	/**
	 * Determine whether the report should be visible in the client user interface or not. This is done by attempting to
	 * obtain the {@link EditorialSARClientService}, which is only registered by the {@link EditorialSARPermanentPlugin} if
	 * the {@link EditorialSARProjectApp} component has been added to the current project.
	 *
	 * @return {@code true} if the report should be visible, otherwise {@code false}.
	 */
	@Override
	public boolean isVisible() {
		try {
			final ServicesBroker broker = _context.requireSpecialist(ServicesBroker.TYPE);
			broker.getService(EditorialSARClientService.class);
			return true;
		} catch (final ServiceNotFoundException e) {
			return false;
		}
	}


	/**
	 * Provide the report's title (i.e., display name).
	 *
	 * @return A {@link String} containing the report's title.
	 */
	@Override
	public String getTitle() {
		return _resourceBundle.getString("report.label");
	}


	/**
	 * Provide the report's icon, optionally considering the report button's current state.
	 *
	 * @param selected The selected-state of the icon.
	 * @return An {@link Image} based on an {@link ImageIcon} resource (SiteArchitect) or a URL (ContentCreator).
	 */
	@Nullable
	@Override
	public Image<?> getIcon(final boolean selected) {
		if (!_context.is(BaseContext.Env.WEBEDIT)) {
			if (selected) {
				return Image.Factory.fromIcon(new javax.swing.ImageIcon(EditorialSARReportPlugin.class.getResource("report_EditorialSAR_active.png")));
			} else {
				return Image.Factory.fromIcon(new javax.swing.ImageIcon(EditorialSARReportPlugin.class.getResource("report_EditorialSAR_normal.png")));
			}
		} else {
			if (selected) {
				return Image.Factory.fromUrl("mod-editorialsar/media/report_EditorialSAR_active.png");
			} else {
				return Image.Factory.fromUrl("mod-editorialsar/media/report_EditorialSAR_normal.png");
			}
		}
	}


	/**
	 * Provide the report parameters used to draw the report's filter panel.
	 *
	 * @return A {@link List} of {@link Parameter} objects.
	 */
	@NotNull
	@Override
	public List<Parameter<?>> getParameter() {
		return Arrays.<Parameter<?>>asList(
				_searchWords,
				_replacementWords,
				_caseSensitive,
				_searchScope,
				_searchLanguages
		);
	}


	/**
	 * Provide a {@link DataProvider} compatible with this report plug-in (i.e., with the same parameterization as this class).
	 *
	 * @return A {@link DataProvider}.
	 */
	@NotNull
	@Override
	public DataProvider<ReportEntry> createDataProvider() {
		final Map<String, Parameter<?>> parameters = new HashMap<String, Parameter<?>>();
		parameters.put("search_words", _searchWords);
		parameters.put("replacement_words", _replacementWords);
		parameters.put("case_sensitive", _caseSensitive);
		parameters.put("search_scope", _searchScope);
		parameters.put("search_languages_abbreviation", _searchLanguages);
		return new EditorialSARDataProvider(_context, parameters);
	}


	/**
	 * Provide a {@link DataRenderer} compatible with this report plug-in (i.e., with the same parameterization as this class).
	 *
	 * @return A {@link DataRenderer}.
	 */
	@NotNull
	@Override
	public DataRenderer<ReportEntry> createDataRenderer() {
		return new EditorialSARDataRenderer(_context, _resourceBundle);
	}


	/**
	 * (Optional) Provide a {@link TransferHandler} compatible with this report plug-in (i.e., with the same parameterization as this class).
	 *
	 * @return A {@link TransferHandler}.
	 */
	@Nullable
	@Override
	public TransferHandler<ReportEntry> createTransferHandler() {
		return null;
	}


	/**
	 * Provide {@link ReportItem}s compatible with this report plug-in (i.e., with the same parameterization as this class).
	 *
	 * @return A {@link Collection} of {@link ReportItem} objects, or an empty collection.
	 */
	@NotNull
	@Override
	public Collection<? extends ReportItem<ReportEntry>> getItems() {
		if (!_context.is(BaseContext.Env.WEBEDIT)) {
			return Arrays.asList(
					new ReplaceAllJavaClientExecutableReportItem(_resourceBundle)
			);
		} else {
			return Arrays.asList(
					new ReplaceAllWebeditExecutableReportItem(_resourceBundle)
			);
		}
	}


	/**
	 * (Optional) Provide a {@link ReportItem} compatible with this report plug-in (i.e., with the same parameterization as this class).
	 *
	 * @return A {@link ReportItem} or {@code null}.
	 */
	@Nullable
	@Override
	public ReportItem<ReportEntry> getDefaultItem() {
		if (!_context.is(BaseContext.Env.WEBEDIT)) {
			return new NavigateToElementJavaClientExecutableReportItem(_resourceBundle);
		} else {
			return new NavigateToElementWebeditExecutableReportItem(_resourceBundle);
		}
	}


	/**
	 * Basic initialization: identify the best display language (based on client program), obtain the resource bundle that
	 * best fits that display language, then build parameters using that resource bundle.
	 *
	 * @param context The context the plug-in lives in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;
		_displayLanguage =
				_context.is(BaseContext.Env.WEBEDIT) ?
						_context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage() :
						_context.requireSpecialist(UIAgent.TYPE).getDisplayLanguage();
		_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, _displayLanguage.getLocale());

		buildParameters();
	}


	/**
	 * Build a set of parameters, obeying the functional differences of ContentCreator and SiteArchitect.
	 */
	private void buildParameters() {
		// Text parameters for search and replacement strings
		_searchWords = Parameter.Factory.createText("search_words", _resourceBundle.getString("report.filter.searchbox.hint") + "...", null);
		_replacementWords = Parameter.Factory.createText("replacement_words", _resourceBundle.getString("report.filter.replacebox.hint") + "...", null);

		// Select parameter and options to limit search by element type.
		final List<ParameterSelect.SelectItem> storeTypes = new ArrayList<ParameterSelect.SelectItem>();
		if (_context.is(BaseContext.Env.WEBEDIT)) {
			// ContentCreator: include an option that searches "web pages" (i.e., Page and Section elements, resulting in combined, Page-based report entries).
			storeTypes.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.store_types.web_pages"), SearchConfiguration.Scope.WEB_PAGES.name()));
			storeTypes.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.store_types.datasets"), SearchConfiguration.Scope.DATASETS.name()));
			storeTypes.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.store_types.media"), SearchConfiguration.Scope.MEDIA.name()));
			_searchScope = Parameter.Factory.createSelect("search_scope", storeTypes, SearchConfiguration.Scope.WEB_PAGES.name());
		} else {
			// SiteArchitect: include an option that searches Page and Section elements, resulting in individual report entries.
			storeTypes.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.store_types.pages_and_sections"), SearchConfiguration.Scope.PAGES_AND_SECTIONS.name()));
			storeTypes.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.store_types.datasets"), SearchConfiguration.Scope.DATASETS.name()));
			storeTypes.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.store_types.media"), SearchConfiguration.Scope.MEDIA.name()));
			storeTypes.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.store_types.templates"), SearchConfiguration.Scope.TEMPLATES.name()));
			_searchScope = Parameter.Factory.createSelect("search_scope", storeTypes, SearchConfiguration.Scope.PAGES_AND_SECTIONS.name());
		}

		// Compile a list of project languages (and an option for "all" languages), then create a select filter with relevant options.
		final List<ParameterSelect.SelectItem> searchLanguages = new ArrayList<ParameterSelect.SelectItem>();
		final StoreAgent storeAgent = _context.requireSpecialist(StoreAgent.TYPE);
		final Store pageStore = storeAgent.getStore(Store.Type.PAGESTORE, false);
		final Project project = pageStore.getProject();
		final List<Language> projectLanguages = project.getLanguages();
		searchLanguages.add(Parameter.Factory.createSelectItem(_resourceBundle.getString("report.filter.languages.all_languages"), "ALL"));
		for (final Language projectLanguage : projectLanguages) {
			String languageName = projectLanguage.getDisplayName(_displayLanguage);
			if (languageName == null || languageName.isEmpty()) {
				languageName = projectLanguage.getAbbreviation();
			}
			searchLanguages.add(Parameter.Factory.createSelectItem(languageName, projectLanguage.getAbbreviation()));
		}
		_searchLanguages = Parameter.Factory.createSelect("search_languages_abbreviation", searchLanguages, "ALL");

		// Boolean filter to control case-sensitivity of search word matching.
		_caseSensitive = Parameter.Factory.createBoolean("case_sensitive", _resourceBundle.getString("report.filter.case_sensitive"), true);
	}


	/**
	 * Clean-up tasks performed as object instances of this class are destroyed.
	 */
	@Override
	public void tearDown() {
	}
}