package de.espirit.firstspirit.opt.example.editorialsar.report.entry;

import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement.DataProviderStoreElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * Report-internal representation of a {@link Dataset} report entry.
 */
public class DatasetReportEntry extends ReportEntry {

	private final Dataset _dataset;


	/**
	 * Constructor.
	 *
	 * See {@link ReportEntry#ReportEntry(EditorialSARClientService, SearchConfiguration)} for the super-class constructor.
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 * @param searchConfiguration The {@link SearchConfiguration}.
	 * @param dataset The {@link Dataset} element this report entry will represent.
	 */
	public DatasetReportEntry(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Dataset dataset) {
		super(clientService, searchConfiguration);
		_dataset = dataset;
	}


	/**
	 * See {@link ReportEntry#getElement()}.
	 */
	@NotNull
	@Override
	public Dataset getElement() {
		return _dataset;
	}


	/**
	 * See {@link ReportEntry#getWebeditTargetElement()}.
	 *
	 * ContentCreator will identify the best way to display a {@link Dataset} element in the preview, so this method simply returns the dataset.
	 */
	@Nullable
	@Override
	public IDProvider getWebeditTargetElement() {
		return _dataset;
	}


	/**
	 * See {@link ReportEntry#doSearch()}.
	 */
	@Override
	public int doSearch() throws IllegalStateException {
		// Check if the search settings contain enough information to conduct a search.
		if (_searchConfiguration.isValidForSearch()) {
			_results.clear();
			int count = 0;

			// Conduct a search with the given search configuration.
			final DataProviderStoreElementSearchAdapter dataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, _dataset);
			_results.addAll(dataProviderSearchAdapter.search());

			// Count the number of results (granularity: editor/field).
			for (final SearchResult result : _results) {
				count += result.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a search without valid search settings.");
		}
	}


	/**
	 * See {@link ReportEntry#doReplace()}.
	 */
	@Override
	public int doReplace() throws IllegalStateException {
		// Check if the search settings contain enough information to conduct a replacement.
		if (_searchConfiguration.isValidForReplace()) {
			int count = 0;
			final DataProviderStoreElementSearchAdapter dataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, _dataset);
			for (final SearchResult searchResult : dataProviderSearchAdapter.replace()) {
				count += searchResult.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a replace without valid search settings.");
		}
	}
}