package de.espirit.firstspirit.opt.example.editorialsar.report.entry;

import de.espirit.firstspirit.access.ReferenceEntry;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.globalstore.GCASection;
import de.espirit.firstspirit.access.store.pagestore.Content2Section;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.access.store.pagestore.SectionReference;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement.DataProviderStoreElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;


/**
 * Report-internal representation of a {@link Page} report entry.
 *
 * This report entry may contain a list of {@link Section} elements as well for ContentCreator compatibility.
 */
public class PageReportEntry extends ReportEntry {

	private final Page _page;
	private final List<Section<?>> _sections = new ArrayList<Section<?>>();


	/**
	 * Constructor.
	 *
	 * See {@link ReportEntry#ReportEntry(EditorialSARClientService, SearchConfiguration)} for the super-class constructor.
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 * @param searchConfiguration The {@link SearchConfiguration}.
	 * @param page The {@link Page} element this report entry will represent.
	 */
	public PageReportEntry(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Page page) {
		super(clientService, searchConfiguration);
		_page = page;
	}


	/**
	 * Identify {@link Section} elements which are child elements of {@link #_page} and contain the search words of the current {@link #_searchConfiguration}.
	 * Add matching sections to {@link #_sections}.
	 */
	public void findAndAddSections() {
		for (final Section<?> section : _page.getChildren(Section.class, true)) {
			if (!(section instanceof Content2Section) && !(section instanceof GCASection) && !(section instanceof SectionReference)) {
				if (section.getPermission().canChange()) {
					final DataProviderStoreElementSearchAdapter sectionSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, section);
					final List<SearchResult> sectionSearchResults = sectionSearchAdapter.search();
					if (!sectionSearchResults.isEmpty()) {
						_sections.add(section);
					}
				}
			}
		}
	}


	/**
	 * Provide a list of {@link Section} elements associated with this {@link Page} report entry.
	 *
	 * @return A {@link List} of {@link Section} objects.
	 */
	@NotNull
	public List<Section<?>> getSections() {
		return _sections;
	}


	/**
	 * See {@link ReportEntry#getElement()}.
	 */
	@NotNull
	@Override
	public Page getElement() {
		return _page;
	}


	/**
	 * See {@link ReportEntry#getWebeditTargetElement()}.
	 *
	 * Attempt to identify a {@link PageRef} element which references this report entry's {@link Page} element. If one
	 * is available, return it; otherwise, return {@code null}.
	 */
	@Nullable
	@Override
	public PageRef getWebeditTargetElement() {
		final ReferenceEntry inboundReferences[] = _page.getIncomingReferences();
		for (final ReferenceEntry inboundReference : inboundReferences) {
			if (inboundReference.getType() == ReferenceEntry.SITE_STORE_REFERENCE) {
				final IDProvider referenceElement = inboundReference.getReferencedElement();
				if (referenceElement instanceof PageRef && _page.equals(((PageRef) referenceElement).getPage())) {
					return (PageRef) referenceElement;
				}
			}
		}
		return null;
	}


	/**
	 * See {@link ReportEntry#doSearch()}.
	 */
	@Override
	public int doSearch() {
		if (_searchConfiguration.isValidForSearch()) {
			_results.clear();
			int count = 0;
			final DataProviderStoreElementSearchAdapter dataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, _page);
			_results.addAll(dataProviderSearchAdapter.search());
			for (final Section<?> section : _sections) {
				final DataProviderStoreElementSearchAdapter sectionDataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, section);
				_results.addAll(sectionDataProviderSearchAdapter.search());
			}

			for (final SearchResult result : _results) {
				count += result.getResultCount();
			}

			return count;
		} else {
			throw new IllegalStateException("Attempt to run a search without valid search settings.");
		}
	}


	/**
	 * See {@link ReportEntry#doReplace()}.
	 */
	@Override
	public int doReplace() throws IllegalStateException {
		if (_searchConfiguration.isValidForReplace()) {
			int count = 0;
			final DataProviderStoreElementSearchAdapter dataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, _page);
			for (final SearchResult searchResult : dataProviderSearchAdapter.replace()) {
				count += searchResult.getResultCount();
			}
			for (final Section<?> section : _sections) {
				final DataProviderStoreElementSearchAdapter sectionDataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, section);
				for (final SearchResult searchResult : sectionDataProviderSearchAdapter.replace()) {
					count += searchResult.getResultCount();
				}
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a replace without valid search settings.");
		}
	}
}