package de.espirit.firstspirit.opt.example.editorialsar.report.entry;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;


/**
 * Report-internal representation of a report entry.
 *
 * This abstract class implements functionality common to the more specific sub-classes.
 */
public abstract class ReportEntry {

	protected final EditorialSARClientService _clientService;
	protected final SearchConfiguration _searchConfiguration;

	protected final List<SearchResult> _results = new ArrayList<SearchResult>();


	/**
	 * Constructor.
	 *
	 * Generate a random {@link UUID} which uniquely identifies a single object instance of this type
	 * and obtain the current search settings from the client service.
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 */
	protected ReportEntry(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration) {
		_clientService = clientService;
		_searchConfiguration = searchConfiguration;
	}


	/**
	 * Provide a list of unprocessed results (i.e., no replacement has occurred yet).
	 *
	 * @return A {@link List} of unprocessed search results.
	 */
	@NotNull
	public List<SearchResult> getResults() {
		return _results;
	}


	/**
	 * Provide the count of unprocessed search results. This count is on an editor/field level.
	 *
	 * @return The count of unprocessed search results.
	 */
	public int getResultCount() {
		return _results.size();
	}


	/**
	 * Provide the count of occurrences of the search string in the report entry's object.
	 *
	 * @return The count of occurrences of the search string.
	 */
	public int getHitCount() {
		int count = 0;
		for (final SearchResult searchResult : _results) {
			count += searchResult.getResultCount();
		}
		return count;
	}


	/**
	 * Provide a list of {@link Language}s that still have unreplaced search results.
	 *
	 * @return A {@link List} of {@link Language} objects.
	 */
	@NotNull
	public List<Language> getResultLanguages() {
		final List<Language> languages = new ArrayList<Language>();
		for (final SearchResult searchResult : _results) {
			final Language language = searchResult.getLanguage();
			if (!languages.contains(language)) {
				languages.add(language);
			}
		}
		return languages;
	}


	@NotNull
	public SearchConfiguration getSearchConfiguration() {
		return _searchConfiguration;
	}


	/**
	 * Provide the element which this report entry object represents.
	 *
	 * @return An {@link IDProvider}.
	 */
	@NotNull
	public abstract IDProvider getElement();


	/**
	 * Identify and provide an element usable for display in ContentCreator's preview (i.e., a {@link PageRef} or {@link Dataset} element).
	 *
	 * @return An {@link IDProvider}, or {@code null} if no target element can be identified.
	 */
	@Nullable
	public abstract IDProvider getWebeditTargetElement();


	/**
	 * Perform a search operation.
	 *
	 * @return The number of occurrences of the {@link #_searchConfiguration}'s search string.
	 * @throws IllegalStateException
	 */
	public abstract int doSearch() throws IllegalStateException;


	/**
	 * Perform a replace operation.
	 *
	 * @return The number of occurrences of the {@link #_searchConfiguration}'s search string that have been replaced with the replacement string.
	 * @throws IllegalStateException
	 */
	public abstract int doReplace() throws IllegalStateException;
}