package de.espirit.firstspirit.opt.example.editorialsar.report.entry;

import de.espirit.firstspirit.access.ReferenceEntry;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.pagestore.Body;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement.DataProviderStoreElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * Report-internal representation of a {@link Section} report entry.
 */
public class SectionReportEntry extends ReportEntry {

	private final Section<?> _section;


	/**
	 * Constructor.
	 *
	 * See {@link ReportEntry#ReportEntry(EditorialSARClientService, SearchConfiguration)} for the super-class constructor.
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 * @param searchConfiguration The {@link SearchConfiguration}.
	 * @param section The {@link Section} element this report entry will represent.
	 */
	public SectionReportEntry(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Section<?> section) {
		super(clientService, searchConfiguration);
		_section = section;
	}


	/**
	 * See {@link ReportEntry#getElement()}.
	 */
	@NotNull
	@Override
	public Section<?> getElement() {
		return _section;
	}


	/**
	 * See {@link ReportEntry#getWebeditTargetElement()}.
	 *
	 * Obtain the {@link Page} element in the parent path of this report entry's {@link Section} element. Then, attempt
	 * to identify a {@link PageRef} element which references that page element. If one is available, return it;
	 * otherwise, return {@code null}.
	 */
	@Nullable
	@Override
	public IDProvider getWebeditTargetElement() {
		final IDProvider body = _section.getParent();
		if (body instanceof Body) {
			final IDProvider page = body.getParent();
			if (page instanceof Page) {
				final ReferenceEntry inboundReferences[] = page.getIncomingReferences();
				for (final ReferenceEntry inboundReference : inboundReferences) {
					if (inboundReference.getType() == ReferenceEntry.SITE_STORE_REFERENCE) {
						final IDProvider referenceElement = inboundReference.getReferencedElement();
						if (referenceElement instanceof PageRef && page.equals(((PageRef) referenceElement).getPage())) {
							return referenceElement;
						}
					}
				}
			}
		}
		return null;
	}


	/**
	 * See {@link ReportEntry#doSearch()}.
	 */
	@Override
	public int doSearch() throws IllegalStateException {
		if (_searchConfiguration.isValidForSearch()) {
			_results.clear();
			int count = 0;
			final DataProviderStoreElementSearchAdapter dataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, _section);
			_results.addAll(dataProviderSearchAdapter.search());
			for (final SearchResult result : _results) {
				count += result.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a search without valid search settings.");
		}
	}


	/**
	 * See {@link ReportEntry#doReplace()}.
	 */
	@Override
	public int doReplace() throws IllegalStateException {
		if (_searchConfiguration.isValidForReplace()) {
			int count = 0;
			final DataProviderStoreElementSearchAdapter dataProviderSearchAdapter = new DataProviderStoreElementSearchAdapter(_clientService, _searchConfiguration, _section);
			for (final SearchResult searchResult : dataProviderSearchAdapter.replace()) {
				count += searchResult.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a replace without valid search settings.");
		}
	}
}