package de.espirit.firstspirit.opt.example.editorialsar.report.item;

import de.espirit.firstspirit.access.project.TemplateSet;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.pagestore.DataProvider;
import de.espirit.firstspirit.agency.HighlightAgent;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.agency.UIViewAgent;
import de.espirit.firstspirit.agency.ViewFocus;
import de.espirit.firstspirit.client.ClientUtil;
import de.espirit.firstspirit.client.plugin.report.JavaClientExecutableReportItem;
import de.espirit.firstspirit.client.plugin.report.ReportContext;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.ReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.TemplateReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.FormDataSearchResult;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.MediaSearchResult;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.TemplateSearchResult;
import de.espirit.firstspirit.ui.operations.DisplayElementOperation;
import de.espirit.firstspirit.ui.operations.OpenElementDataFormOperation;
import de.espirit.firstspirit.ui.operations.OpenElementMetaFormOperation;
import de.espirit.firstspirit.ui.views.View;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.Icon;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.ResourceBundle;
import java.util.regex.Pattern;


/**
 * Executable report item for SiteArchitect that performs navigation to the report entry's store element and highlights
 * the current search settings' search string, stepping through editors/fields and project languages.
 */
public class NavigateToElementJavaClientExecutableReportItem implements JavaClientExecutableReportItem<ReportEntry> {

	private ReportEntry _reportEntry;
	private final LinkedList<SearchResult> _searchResults = new LinkedList<SearchResult>();
	private ListIterator<SearchResult> _searchResultsIterator;

	private final ResourceBundle _resourceBundle;


	public NavigateToElementJavaClientExecutableReportItem(@NotNull final ResourceBundle resourceBundle) {
		_resourceBundle = resourceBundle;
	}


	@Nullable
	@Override
	public Icon getIcon(@NotNull final ReportContext<ReportEntry> context) {
		return new javax.swing.ImageIcon(NavigateToElementJavaClientExecutableReportItem.class.getResource("item_NavigateToElement_normal.png"));
	}


	@NotNull
	@Override
	public String getLabel(@NotNull final ReportContext<ReportEntry> context) {
		return _resourceBundle.getString("report.item.navigate_to.label");
	}


	@Override
	public void execute(@NotNull final ReportContext<ReportEntry> context) {
		Boolean newElement = false;

		// Persist the report entry object so it may be reused between calls of this method.
		if (_reportEntry == null || !_reportEntry.getElement().equals(context.getObject().getElement()) || !_reportEntry.getSearchConfiguration().equals(context.getObject().getSearchConfiguration())) {
			_reportEntry = context.getObject();
			newElement = true;
		}

		if (newElement || _searchResults.isEmpty()) {
			// If no search results are available yet, perform initialization, clearing the persisted search results list and obtaining search results from the report entry.
			_searchResults.clear();
			_searchResults.addAll(_reportEntry.getResults());
			_searchResultsIterator = _searchResults.listIterator();
		} else {
			// If search results are available already, check if the iterator is at the last list position. If so, loop back to the beginning of the list by obtaining a new iterator.
			if (!_searchResultsIterator.hasNext()) {
				_searchResultsIterator = _searchResults.listIterator();
			}
		}

		final IDProvider element = _reportEntry.getElement();

		// Conditional to decide whether navigation has to consider special views (templates) or use simple, language-based navigation (all other elements).
		if (context.getObject() instanceof TemplateReportEntry) {

			/*
			 *  Warning! This block uses non-API functionality to navigate to specific tabs in
			 *  template views in SiteArchitect's workspace pane.

			 *  Non-API classes/interfaces:
			 *  - de.espirit.firstspirit.agency.UIViewAgent
			 *  - de.espirit.firstspirit.agency.ViewFocus
			 *  - de.espirit.firstspirit.client.ClientUtil
			 *  - de.espirit.firstspirit.ui.views.View
			 *
			 *  Use of these classes/interfaces is not supported by e-Spirit and only included here for convenient tab switching in template views.
			 */

			final UIViewAgent uiViewAgent = context.requireSpecialist(UIViewAgent.TYPE);
			final View.Identifier viewIdentifier;

			final SearchResult searchResult = _searchResultsIterator.next();
			if (searchResult instanceof TemplateSearchResult) {
				final TemplateSearchResult.Tab templateTab = ((TemplateSearchResult) searchResult).getTab();
				switch (templateTab) {
					case PROPERTIES:
						viewIdentifier = uiViewAgent.getIdentifier(UIViewAgent.Identity.PROPERTIES);
						break;
					case GOM_SOURCE:
						viewIdentifier = uiViewAgent.getIdentifier(UIViewAgent.Identity.FORM);
						break;
					case RULESET_DEFINITION:
						viewIdentifier = uiViewAgent.getIdentifier(UIViewAgent.Identity.RULES);
						break;
					case SNIPPET:
						viewIdentifier = uiViewAgent.getIdentifier(UIViewAgent.Identity.SNIPPETS);
						break;
					case CHANNEL:
						final TemplateSet templateSet = ((TemplateSearchResult) searchResult).getTemplateSet();
						if (templateSet != null) {
							viewIdentifier = uiViewAgent.getIdentifier(templateSet);
						} else {
							// The search result may erroneously contain a null-value template set object. If that is so,
							// use navigation to the template's Properties tab as a fall-back.
							viewIdentifier = uiViewAgent.getIdentifier(UIViewAgent.Identity.PROPERTIES);
						}
						break;
					default:
						viewIdentifier = uiViewAgent.getIdentifier(UIViewAgent.Identity.PROPERTIES);
				}
				final ViewFocus viewFocus = uiViewAgent.createFocus(viewIdentifier, null);
				ClientUtil.show(element).focusOn(viewFocus).execute();
			}
		} else {
			final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);

			// Obtain the next item in the search results list.
			final SearchResult searchResult = _searchResultsIterator.next();

			// Navigation and content highlighting needs to be done differently, depending on the search result's type (data form, metadata form, media, etc.).
			if (searchResult instanceof FormDataSearchResult && !((FormDataSearchResult) searchResult).isMetadataForm()) {
				// Data form.
				// Use an OpenElementDataFormOperation to navigate to the element in the search result's indicated language, highlighting the indicated editor.
				final OpenElementDataFormOperation openElementDataFormOperation = operationAgent.getOperation(OpenElementDataFormOperation.TYPE);
				if (openElementDataFormOperation != null) {
					openElementDataFormOperation.setLanguage(searchResult.getLanguage());
					openElementDataFormOperation.setFormData(((DataProvider) element).getFormData());
					openElementDataFormOperation.setField(((FormDataSearchResult) searchResult).getEditorName());
					openElementDataFormOperation.perform((DataProvider) element);
				}
			} else if (searchResult instanceof FormDataSearchResult && ((FormDataSearchResult) searchResult).isMetadataForm() && _reportEntry.getElement().hasMeta()) {
				// Metadata form.
				// Use an OpenElementMetaFormOperation to navigate to the element's metadata tab, highlighting the editor indicated by the search result object.
				final FormData metaFormData = element.getMetaFormData();
				if (metaFormData != null) {
					final OpenElementMetaFormOperation openElementMetaFormOperation = operationAgent.getOperation(OpenElementMetaFormOperation.TYPE);
					if (openElementMetaFormOperation != null) {
						openElementMetaFormOperation.setFormData(metaFormData);
						openElementMetaFormOperation.setField(((FormDataSearchResult) searchResult).getEditorName());
						openElementMetaFormOperation.perform(element);
					}
				}
			} else if (searchResult instanceof MediaSearchResult) {
				// Media.
				// Use a DisplayElementOperation to navigate to the Media element.
				if (((MediaSearchResult) searchResult).getField() == MediaSearchResult.Field.DESCRIPTION || ((MediaSearchResult) searchResult).getField() == MediaSearchResult.Field.FILENAME) {
					final DisplayElementOperation displayElementOperation = operationAgent.getOperation(DisplayElementOperation.TYPE);
					if (displayElementOperation != null) {
						displayElementOperation.setLanguage(searchResult.getLanguage());
						displayElementOperation.perform(element);
					}
				}
			}
		}

		// Lastly, configure highlighting to mark the search string used to find the report entry's element, and potentially the replacement string as well.
		final HighlightAgent highlightAgent = context.requireSpecialist(HighlightAgent.TYPE);
		final SearchConfiguration searchConfiguration = _reportEntry.getSearchConfiguration();

		final StringBuilder highlightSearchWordPattern = new StringBuilder();
		if (!searchConfiguration.getCaseSensitive()) {
			highlightSearchWordPattern.append("(?i)");
		}
		highlightSearchWordPattern.append(Pattern.quote(searchConfiguration.getSearchWords()));

		if (searchConfiguration.isValidForReplace()) {
			highlightAgent.highlightPattern(highlightSearchWordPattern.toString() + '|' + "(?-i)" + Pattern.quote(searchConfiguration.getReplacementWords()));
		} else {
			highlightAgent.highlightPattern(highlightSearchWordPattern.toString());
		}
	}


	@Override
	public boolean isVisible(@NotNull final ReportContext<ReportEntry> context) {
		return true;
	}


	@Override
	public boolean isEnabled(@NotNull final ReportContext<ReportEntry> context) {
		return true;
	}
}