package de.espirit.firstspirit.opt.example.editorialsar.report.item;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.ElementDeletedException;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.LockException;
import de.espirit.firstspirit.access.store.pagestore.Body;
import de.espirit.firstspirit.access.store.pagestore.DataProvider;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.agency.FormValidationReport;
import de.espirit.firstspirit.agency.HighlightAgent;
import de.espirit.firstspirit.agency.MultiFormValidationReport;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.agency.ValidationAgent;
import de.espirit.firstspirit.client.plugin.report.JavaClientExecutableReportItem;
import de.espirit.firstspirit.client.plugin.report.ReportContext;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.ReportEntry;
import de.espirit.firstspirit.ui.operations.DisplayElementOperation;
import de.espirit.firstspirit.ui.operations.OpenElementDataFormOperation;
import de.espirit.firstspirit.ui.operations.RequestOperation;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.Icon;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.regex.Pattern;


/**
 * Executable report item for SiteArchitect that starts search word replacement on a report entry and performs navigation
 * to that report entry's element to highlight changes.
 */
public class ReplaceAllJavaClientExecutableReportItem implements JavaClientExecutableReportItem<ReportEntry> {

	private final ResourceBundle _resourceBundle;


	public ReplaceAllJavaClientExecutableReportItem(@NotNull final ResourceBundle resourceBundle) {
		_resourceBundle = resourceBundle;
	}


	@Nullable
	@Override
	public Icon getIcon(@NotNull final ReportContext<ReportEntry> context) {
		return new javax.swing.ImageIcon(ReplaceAllJavaClientExecutableReportItem.class.getResource("item_ReplaceText_normal.png"));
	}


	@NotNull
	@Override
	public String getLabel(@NotNull final ReportContext<ReportEntry> context) {
		return _resourceBundle.getString("report.item.replace_all.label");
	}


	@Override
	public void execute(@NotNull final ReportContext<ReportEntry> context) {
		final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);

		final ReportEntry reportEntry = context.getObject();
		final IDProvider element = reportEntry.getElement();

		IDProvider lockElement = element;
		if (lockElement instanceof Section) {
			lockElement = lockElement.getParent();
		}
		if (lockElement instanceof Body) {
			lockElement = lockElement.getParent();
		}

		try {
			// Lock the lockElement for upcoming edits. If the lockElement is a Page object, perform a recursive lock (second parameter of IDProvider#setLock(boolean, boolean) to include Section elements).
			if (lockElement != null) {
				lockElement.setLock(true, lockElement instanceof Page);
			} else {
				throw new IllegalStateException("Could not identify the correct lockable element.");
			}

			reportEntry.doReplace();

			// Handle saving the element.
			if (element instanceof DataProvider) {
				// For DataProvider elements, we must validate the element in order to check for potentially invalid form data
				// which may have been caused by the replacement text inserted into GOM editors.
				final ValidationAgent validationAgent = context.requireSpecialist(ValidationAgent.TYPE);
				final MultiFormValidationReport multiFormValidationReport = validationAgent.validate(element, ValidationAgent.ValidationScope.SAVE);
				boolean validationIssues = false;
				for (final Language validationLanguage : multiFormValidationReport.getLanguages(context)) {
					validationIssues = validationIssues || multiFormValidationReport.getProblems(validationLanguage) != null;
				}
				validationIssues = validationIssues || multiFormValidationReport.getProblemsForMetaData() != null;

				if (!validationIssues) {
					boolean unlock = true;
					boolean elementExists = true;

					// The element is valid. Save it and unlock the lockElement (applying recursive unlock if the lockElement is a Page object).
					try {
						element.save("Editorial Search and Replace: Replacement Process");
						lockElement.setLock(false, lockElement instanceof Page);
					} catch (final LockException e) {
						context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") could not be unlocked after replacement process.");
						unlock = false;
					} catch (final ElementDeletedException e) {
						context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") seems to have been deleted.");
						elementExists = false;
					} finally {
						if (elementExists) {
							// Obtain a DisplayElementOperation object and navigate to the report entry's element.
							final DisplayElementOperation displayElementOperation = operationAgent.getOperation(DisplayElementOperation.TYPE);
							if (displayElementOperation != null) {
								displayElementOperation.perform(reportEntry.getElement());
							}
						}

						// The element is still locked for editing. Notify the user that a manual save/unlock procedure is recommended.
						if (!unlock) {
							final RequestOperation requestOperation = operationAgent.getOperation(RequestOperation.TYPE);
							if (requestOperation != null) {
								requestOperation.setKind(RequestOperation.Kind.ERROR);
								requestOperation.addOk();
								requestOperation.setTitle(_resourceBundle.getString("report.item.replace_all.unlock_failed.title"));
								requestOperation.perform(_resourceBundle.getString("report.item.replace_all.unlock_failed.text"));
							}
						}
					}
				} else {
					// The element contains invalid form data in either data or metadata form, or both. Navigate to the element, display a language with form issues,
					// and inform the user so problems can be corrected manually.
					final OpenElementDataFormOperation openElementDataFormOperation = operationAgent.getOperation(OpenElementDataFormOperation.TYPE);
					final RequestOperation requestOperation = operationAgent.getOperation(RequestOperation.TYPE);
					if (openElementDataFormOperation != null) {
						Language firstProblemLanguage = null;
						for (final Language problemLanguage : multiFormValidationReport.getLanguages(context)) {
							final FormValidationReport problems = multiFormValidationReport.getProblems(problemLanguage);
							if (problems != null) {
								final Set<String> problemGadgets = problems.getGadgets();
								if (!problemGadgets.isEmpty()) {
									firstProblemLanguage = problemLanguage;
									break;
								}
							}
						}
						if (firstProblemLanguage != null) {
							openElementDataFormOperation.setLanguage(firstProblemLanguage);
						}
						openElementDataFormOperation.setFormData(((DataProvider) element).getFormData());
						openElementDataFormOperation.setOpenEditable(true);
						openElementDataFormOperation.perform((DataProvider) element);
						if (requestOperation != null) {
							requestOperation.setKind(RequestOperation.Kind.ERROR);
							requestOperation.setTitle(_resourceBundle.getString("report.item.replace_all.replace_failed.title"));
							requestOperation.addOk();
							requestOperation.perform(_resourceBundle.getString("report.item.replace_all.replace_failed.text"));
						}
					}
				}
			} else {
				boolean unlock = true;
				boolean elementExists = true;

				// The element is not a DataProvider - it can be saved directly without validation.
				try {
					element.save("Editorial Search and Replace: Replacement Process");
					element.setLock(false, false);
				} catch (final LockException e) {
					context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") could not be unlocked after replacement process.");
					unlock = false;
				} catch (final ElementDeletedException e) {
					context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") seems to have been deleted during replacement process.");
					elementExists = false;
				} finally {
					if (elementExists) {
						// Obtain a DisplayElementOperation object and navigate to the report entry's element.
						final DisplayElementOperation displayElementOperation = operationAgent.getOperation(DisplayElementOperation.TYPE);
						if (displayElementOperation != null) {
							displayElementOperation.perform(reportEntry.getElement());
						}
					}

					if (!unlock) {
						// The element is still locked for editing. Notify the user that a manual save/unlock procedure is recommended.
						final RequestOperation requestOperation = operationAgent.getOperation(RequestOperation.TYPE);
						if (requestOperation != null) {
							requestOperation.setKind(RequestOperation.Kind.ERROR);
							requestOperation.addOk();
							requestOperation.setTitle(_resourceBundle.getString("report.item.replace_all.unlock_failed.title"));
							requestOperation.perform(_resourceBundle.getString("report.item.replace_all.unlock_failed.text"));
						}
					}
				}
			}
		} catch (final LockException e) {
			context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") could not be locked before replacement process.");
		} catch (final ElementDeletedException e) {
			context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") seems to have been deleted.");
		} finally {
			// Highlight the search and the replacement string in the element's form.
			final String replacementWords = reportEntry.getSearchConfiguration().getReplacementWords();
			if (replacementWords != null && !replacementWords.isEmpty()) {
				final HighlightAgent highlightAgent = context.requireSpecialist(HighlightAgent.TYPE);

				final StringBuilder highlightSearchWordPattern = new StringBuilder();
				if (!reportEntry.getSearchConfiguration().getCaseSensitive()) {
					highlightSearchWordPattern.append("(?i)");
				}
				highlightSearchWordPattern.append(Pattern.quote(reportEntry.getSearchConfiguration().getSearchWords()));

				highlightAgent.highlightPattern(highlightSearchWordPattern.toString() + '|' + "(?-i)" + Pattern.quote(reportEntry.getSearchConfiguration().getReplacementWords()));
			}
		}
	}


	@Override
	public boolean isVisible(@NotNull final ReportContext<ReportEntry> context) {
		// Only display this report item if the search configuration reports it contains enough information for a replacement process.
		return context.getObject().getSearchConfiguration().isValidForReplace();
	}


	@Override
	public boolean isEnabled(@NotNull final ReportContext<ReportEntry> context) {
		// Only enable this report item if the report entry has unprocessed results, and the entry's element is not currently locked.
		return context.getObject().getResultCount() > 0 && !context.getObject().getElement().isLockedOnServer(true);
	}
}