package de.espirit.firstspirit.opt.example.editorialsar.report.item;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.ElementDeletedException;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.LockException;
import de.espirit.firstspirit.access.store.pagestore.Body;
import de.espirit.firstspirit.access.store.pagestore.DataProvider;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.agency.MultiFormValidationReport;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.agency.ValidationAgent;
import de.espirit.firstspirit.client.plugin.report.ReportContext;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.PageReportEntry;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.ReportEntry;
import de.espirit.firstspirit.ui.operations.DisplayElementOperation;
import de.espirit.firstspirit.ui.operations.RequestOperation;
import de.espirit.firstspirit.webedit.plugin.report.WebeditExecutableReportItem;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ResourceBundle;


/**
 * Executable report item for ContentCreator that starts search word replacement on a report entry and performs navigation
 * to that report entry's preview element (e.g. a {@link PageRef}.
 */
public class ReplaceAllWebeditExecutableReportItem implements WebeditExecutableReportItem<ReportEntry> {

	private final ResourceBundle _resourceBundle;


	public ReplaceAllWebeditExecutableReportItem(@NotNull final ResourceBundle resourceBundle) {
		_resourceBundle = resourceBundle;
	}


	@NotNull
	@Override
	public String getLabel(@NotNull final ReportContext<ReportEntry> context) {
		return _resourceBundle.getString("report.item.replace_all.label");
	}


	@Override
	public void execute(@NotNull final ReportContext<ReportEntry> context) {
		final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);

		final ReportEntry reportEntry = context.getObject();
		final IDProvider element = reportEntry.getElement();

		IDProvider lockElement = element;
		if (lockElement instanceof Section) {
			lockElement = lockElement.getParent();
		}
		if (lockElement instanceof Body) {
			lockElement = lockElement.getParent();
		}

		try {
			// Lock the lockElement for upcoming edits. If the lockElement is a Page object, perform a recursive lock (second parameter of IDProvider#setLock(boolean, boolean) to include Section elements).
			if (lockElement != null) {
				lockElement.setLock(true, lockElement instanceof Page);
			} else {
				throw new IllegalStateException("Could not identify the correct lockable element.");
			}

			// Start replacement on the report entry.
			reportEntry.doReplace();

			// Handle saving the element.
			if (element instanceof DataProvider) {
				// For DataProvider elements, we must validate the element in order to check for potentially invalid form data
				// which may have been caused by the replacement text inserted into GOM editors and which will prevent saving the element.
				final ValidationAgent validationAgent = context.requireSpecialist(ValidationAgent.TYPE);
				final MultiFormValidationReport multiFormValidationReport = validationAgent.validate(element, ValidationAgent.ValidationScope.SAVE);
				boolean validationIssues = false;
				for (final Language validationLanguage : multiFormValidationReport.getLanguages(context)) {
					validationIssues = validationIssues || multiFormValidationReport.getProblems(validationLanguage) != null;
				}
				validationIssues = validationIssues || multiFormValidationReport.getProblemsForMetaData() != null;

				// For Page elements, validate child elements of type Section as well.
				if (element instanceof Page) {
					for (final Section<?> section : element.getChildren(Section.class, true)) {
						final MultiFormValidationReport sectionMultiFormValidationReport = validationAgent.validate(section, ValidationAgent.ValidationScope.SAVE);
						for (final Language sectionValidationLanguage : sectionMultiFormValidationReport.getLanguages(context)) {
							validationIssues = validationIssues || sectionMultiFormValidationReport.getProblems(sectionValidationLanguage) != null;
						}
						validationIssues = validationIssues || sectionMultiFormValidationReport.getProblemsForMetaData() != null;
					}
				}

				if (!validationIssues) {
					boolean elementExists = true;

					// The element is valid. Save it.
					try {
						if (reportEntry instanceof PageReportEntry) {
							// Before saving the Page element, save all child Section elements which are considered by this report entry object.
							for (final Section<?> section : ((PageReportEntry) reportEntry).getSections()) {
								section.save("Editorial Search and Replace: Replacement Process");
							}
						}
						element.save("Editorial Search and Replace: Replacement Process");
						lockElement.setLock(false, lockElement instanceof Page);
					} catch (final LockException e) {
						context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") could not be unlocked after replacement process.");
					} catch (final ElementDeletedException e) {
						context.logError("Lock element " + lockElement.getUid() + " (ID: " + element.getLongID() + ") seems to have been deleted.");
						elementExists = false;
					} finally {
						if (elementExists) {
							// Let the user know a replacement occurred. The element will be loaded/reloaded in the preview.
							final IDProvider targetElement = reportEntry.getWebeditTargetElement();
							if (targetElement != null) {
								// Previewable element. Cause navigation to the element.
								final DisplayElementOperation displayElementOperation = operationAgent.getOperation(DisplayElementOperation.TYPE);
								if (displayElementOperation != null) {
									displayElementOperation.perform(targetElement);
								}
							} else {
								// Non-previewable element. Display a message box.
								final RequestOperation requestOperation = operationAgent.getOperation(RequestOperation.TYPE);
								if (requestOperation != null) {
									requestOperation.setKind(RequestOperation.Kind.INFO);
									requestOperation.setTitle(_resourceBundle.getString("report.item.replace_all.replace_successful.title"));
									requestOperation.addOk();
									requestOperation.perform(_resourceBundle.getString("report.item.replace_all.replace_successful.text"));
								}
							}
						}
					}
				} else {
					// The element (in case of a Page element, this includes child Section elements as well) contains invalid form data in either data or metadata form, or both,
					// which prevent saving the element. Navigate to the target element and inform the user so replacement can be conducted manually.
					try {
						if (element instanceof Page) {
							element.setLock(false, true);
						} else {
							element.setLock(false, false);
						}
					} catch (final LockException e) {
						context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") could not be unlocked after failed replacement process.");
					} catch (final ElementDeletedException e) {
						context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") seems to have been deleted.");
					} finally {
						final DisplayElementOperation displayElementOperation = operationAgent.getOperation(DisplayElementOperation.TYPE);
						final IDProvider targetElement = reportEntry.getWebeditTargetElement();
						if (displayElementOperation != null && targetElement != null) {
							displayElementOperation.perform(targetElement);
						}
						final RequestOperation requestOperation = operationAgent.getOperation(RequestOperation.TYPE);
						if (requestOperation != null) {
							requestOperation.setKind(RequestOperation.Kind.ERROR);
							requestOperation.setTitle(_resourceBundle.getString("report.item.replace_all.replace_failed.title"));
							requestOperation.addOk();
							requestOperation.perform(_resourceBundle.getString("report.item.replace_all.replace_failed.text"));
						}
					}
				}
			} else {
				// The element is not a DataProvider - it can be saved directly without validation.
				try {
					element.save("Editorial Search and Replace: Replacement Process");
					element.setLock(false, false);
				} catch (final LockException e) {
					context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") could not be unlocked after replacement process.");
				} catch (final ElementDeletedException e) {
					context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") seems to have been deleted.");
				} finally {
					// Let the user know a replacement occurred. A message box notifying the user of replacement success will be displayed.
					final RequestOperation requestOperation = operationAgent.getOperation(RequestOperation.TYPE);
					if (requestOperation != null) {
						requestOperation.setKind(RequestOperation.Kind.INFO);
						requestOperation.setTitle(_resourceBundle.getString("report.item.replace_all.replace_successful.title"));
						requestOperation.addOk();
						requestOperation.perform(_resourceBundle.getString("report.item.replace_all.replace_successful.text"));
					}
				}
			}
		} catch (final LockException e) {
			context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") could not be locked before replacement process.");
			return;
		} catch (final ElementDeletedException e) {
			context.logError("Element " + element.getUid() + " (ID: " + element.getLongID() + ") seems to have been deleted.");
			return;
		}
	}


	@Nullable
	@Override
	public String getIconPath(@NotNull final ReportContext<ReportEntry> context) {
		return "mod-editorialsar/media/item_ReplaceText_small.png";
	}


	@Override
	public boolean isVisible(@NotNull final ReportContext<ReportEntry> context) {
		// Only display this report item if the search configuration reports it contains enough information for a replacement process.
		return context.getObject().getSearchConfiguration().isValidForReplace();
	}


	@Override
	public boolean isEnabled(@NotNull final ReportContext<ReportEntry> context) {
		// Only enable this report item if the report entry has unprocessed results, and the entry's element is not currently locked.
		return context.getObject().getResultCount() > 0 && !context.getObject().getElement().isLockedOnServer(true);
	}
}