package de.espirit.firstspirit.opt.example.editorialsar.search;

import de.espirit.firstspirit.access.Language;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;


/**
 * Persists and handles search and replacement settings and provided by report filter configuration.
 */
public final class SearchConfiguration {

	private final String _searchWords;
	private final String _replacementWords;
	private final Scope _searchScope;
	private final List<Language> _languages;
	private final boolean _caseSensitive;


	public enum Scope {
		WEB_PAGES,
		PAGES_AND_SECTIONS,
		DATASETS,
		MEDIA,
		TEMPLATES
	}


	public SearchConfiguration(@NotNull final List<Language> languages, @NotNull final String searchWords, @Nullable final String replacementWords, @NotNull final Scope searchScope, final boolean caseSensitive) {
		_searchWords = searchWords;
		_replacementWords = replacementWords;
		_searchScope = searchScope;
		_languages = languages;
		_caseSensitive = caseSensitive;
	}


	/**
	 * Provide the search string.
	 *
	 * @return A {@link String} containing search words.
	 */
	@NotNull
	public String getSearchWords() {
		return _searchWords;
	}


	/**
	 * Provide the replacement string.
	 *
	 * @return A {@link String} containing replacement words.
	 */
	@Nullable
	public String getReplacementWords() {
		return _replacementWords;
	}


	/**
	 * Provide the search scope.
	 *
	 * @return The {@link Scope} of this search configuration.
	 */
	@NotNull
	public Scope getSearchScope() {
		return _searchScope;
	}


	/**
	 * Provide a list of project languages that should be searched.
	 *
	 * @return A {@link List} of {@link Language} objects.
	 */
	@NotNull
	public List<Language> getLanguages() {
		return _languages;
	}


	/**
	 * Indicate if search word matching should be case-sensitive.
	 *
	 * @return {@code true} if searches should be case-sensitive, otherwise {@code false}.
	 */
	public boolean getCaseSensitive() {
		return _caseSensitive;
	}


	/**
	 * Determine whether enough information is available to conduct a search or not.
	 *
	 * At least one language and a search string of at least three characters length must be configured.
	 *
	 * @return {@code true} if search settings are valid to perform a search, otherwise {@code false}.
	 */
	public boolean isValidForSearch() {
		return !_languages.isEmpty() && !_searchWords.isEmpty() && _searchWords.length() >= 3;
	}


	/**
	 * Determine whether enough information is available to conduce a replacement process or not.
	 *
	 * In addition to language and search string, a replacement word of at least three characters length must be configured.
	 *
	 * @return {@code true} if search settings are valid to perform a replacement process, otherwise {@code false}.
	 */
	public boolean isValidForReplace() {
		return isValidForSearch() && _replacementWords != null && !_replacementWords.isEmpty() && _replacementWords.length() >= 3;
	}


	/**
	 * Determine matching identity between the provided {@code object} and the current object instance.
	 *
	 * @param object The object to compare identity with.
	 * @return {@code true} if all important fields match (identity), {@code false} otherwise.
	 */
	public boolean equals(final Object object) {
		return object instanceof SearchConfiguration
				&& _searchWords.equals(((SearchConfiguration) object).getSearchWords())
				&& ((_replacementWords == null && ((SearchConfiguration) object).getReplacementWords() == null)
					|| (_replacementWords != null && _replacementWords.equals(((SearchConfiguration) object).getReplacementWords())))
				&& _searchScope == ((SearchConfiguration) object).getSearchScope()
				&& _languages.equals(((SearchConfiguration) object).getLanguages())
				&& _caseSensitive == ((SearchConfiguration) object).getCaseSensitive();
	}


	@Override
	public int hashCode() {
		int result = _searchWords.hashCode();
		result = 31 * result + _searchScope.hashCode();
		return result;
	}
}