package de.espirit.firstspirit.opt.example.editorialsar.search.adapter;

import de.espirit.firstspirit.access.editor.value.DomNode;
import de.espirit.firstspirit.access.editor.value.DomNodeType;
import de.espirit.firstspirit.access.editor.value.LinkDomNode;
import de.espirit.firstspirit.access.link.Link;
import de.espirit.firstspirit.access.store.templatestore.FormatTemplate;
import de.espirit.firstspirit.access.store.templatestore.LinkTemplate;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Utility class that provides often-used functionality to {@link SearchAdapter} extensions.
 */
public class SearchAdapterUtil {


	/**
	 * Process the given {@code text} using the given {@code searchSettings}, optionally performing a text replacement.
	 *
	 * @param text The text to process.
	 * @param searchSettings The {@link SearchConfiguration} containing search settings such as search/replacement words and case-sensitivity.
	 * @param replace Perform a replacement if {@link true}.
	 * @return A {@link ProcessingResult}.
	 */
	@NotNull
	public static ProcessingResult processText(@NotNull final String text, @NotNull final SearchConfiguration searchSettings, final boolean replace) {
		final String searchWords = searchSettings.getSearchWords();
		final String replacementWords = searchSettings.getReplacementWords();
		final boolean caseSensitive = searchSettings.getCaseSensitive();

		// Check if a replacement should (variable 'replace') and can (method 'searchSettings.isValidForReplace()') be done.
		final boolean reallyReplace = replace && searchSettings.isValidForReplace();

		final int regExpFlags = caseSensitive ? 0 : Pattern.CASE_INSENSITIVE;
		final Pattern regExpPattern = Pattern.compile(Pattern.quote(searchWords), regExpFlags);
		final Matcher regExpMatcher = regExpPattern.matcher(text);

		int count = 0;
		String processedText = text;

		// Count the number of search string occurrences in the text.
		while (regExpMatcher.find()) {
			count++;
		}

		if (reallyReplace) {
			// Replace occurrences of search string with replacement string.
			processedText = regExpMatcher.replaceAll(Matcher.quoteReplacement(replacementWords));
		} else if (replace) {
			// A replace was requested, but search settings don't allow a replacement. Set count to 0 to indicate nothing has been done to the text.
			count = 0;
		}

		return new ProcessingResult(processedText, count, count > 0);
	}


	/**
	 * Process a DOM node, iterating through a given DOM object until nodes of type TEXT or LINK have been found (see {@link DomNodeType}).
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 * @param domRoot The DOM root to process.
	 * @param replace Perform a replacement if {@link true}.
	 * @return A {@link List} of {@link ProcessingResult} objects.
	 */
	@NotNull
	public static List<ProcessingResult> processDomNode(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final DomNode domRoot, final boolean replace) {
		final List<ProcessingResult> processingResults = new ArrayList<ProcessingResult>();
		for (final DomNode child : domRoot.getChildren()) {
			final DomNodeType childNodeType = child.getNodeType();
			if (childNodeType == DomNodeType.TEXT) {
				// Child node is of type 'TEXT'. If this node's format template is valid for search and replace, process it.
				final FormatTemplate childFormatTemplate = child.getFormatTemplate();
				if (childFormatTemplate == null || clientService.isSupportedTemplate(childFormatTemplate)) {
					final String childText = child.toString();
					if (childText != null && !childText.isEmpty()) {
						final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(childText, searchConfiguration, replace);
						if (result.getProcessingCount() > 0) {
							processingResults.add(result);
							if (replace && result.getReplacementStatus()) {
								domRoot.replaceChild(child, result.getProcessedText());
							}
						}
					}
				}
			} else if (childNodeType == DomNodeType.LINK) {
				// Child node is of type 'LINK'. If the link template is valid for search and replace, process the link text (the link's form data itself - i.e. fields other than the link text - will not be processed).
				final Link childLink = ((LinkDomNode) child).getLink();
				final LinkTemplate childLinkTemplate = childLink.getTemplate();
				if (childLinkTemplate != null && clientService.isSupportedTemplate(childLinkTemplate)) {
					final String linkText = childLink.getText();
					final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(linkText, searchConfiguration, replace);
					if (result.getProcessingCount() > 0) {
						processingResults.add(result);
						if (replace && result.getReplacementStatus()) {
							childLink.setText(result.getProcessedText());
						}
					}
				}
			} else {
				processingResults.addAll(processDomNode(clientService, searchConfiguration, child, replace));
			}
		}
		return processingResults;
	}


	/**
	 * A class that stores processing results, including the processed text, count of found or replaced occurrences (depending
	 * on processing settings), and a flag that indicates if the count reflects found or replaced occurrences.
	 */
	public static class ProcessingResult {

		private final String _text;
		private final int _count;
		private final boolean _replacementOccurred;


		ProcessingResult(@NotNull final String text, final int count, final boolean replacementOccurred) {
			_text = text;
			_count = count;
			_replacementOccurred = replacementOccurred && count > 0;
		}


		@NotNull
		public String getProcessedText() {
			return _text;
		}


		public int getProcessingCount() {
			return _count;
		}


		public boolean getReplacementStatus() {
			return _replacementOccurred;
		}
	}
}