package de.espirit.firstspirit.opt.example.editorialsar.search.adapter.formdata;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.templatestore.gom.GomFormElement;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.SearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.gomelement.DomGomElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.gomelement.DomTableGomElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.gomelement.FsListGomElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.gomelement.TextGomElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;


/**
 * Search adapter to perform search and replacement processes on {@link FormData} objects.
 */
public class FormDataSearchAdapter extends SearchAdapter {

	private final Language _language;
	private final FormData _formData;
	private final boolean _isMetadataForm;


	/**
	 * Constructor.
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 * @param language The project {@link Language} to process.
	 * @param formData The {@link FormData} object to process.
	 * @param isMetadataForm Indicates if the {@link FormData} object is a metadata form {@code true} or a normal element form {@code false}.
	 */
	public FormDataSearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Language language, @NotNull final FormData formData, final boolean isMetadataForm) {
		super(clientService, searchConfiguration);
		_language = language;
		_formData = formData;
		_isMetadataForm = isMetadataForm;
	}


	/**
	 * Process the given {@link FormData} object.
	 *
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	@Override
	protected List<SearchResult> process() {
		final List<SearchResult> searchResults = new ArrayList<SearchResult>();

		// Cycle through all the form's form elements.
		for (final GomFormElement gomFormElement : _formData.getForm().forms()) {

			// Match individual form elements by GOM tag and use an appropriate GomElementSearchAdapter sub-class if available.
			final String gomElementTag = gomFormElement.getGomElementTag();
			final String gomElementName = gomFormElement.name();

			// CMS_INPUT_TEXT/CMS_INPUT_TEXTAREA: Use a TextGomElementSearchAdapter for processing.
			if ("CMS_INPUT_TEXT".equals(gomElementTag) || "CMS_INPUT_TEXTAREA".equals(gomElementTag)) {
				final TextGomElementSearchAdapter textGomElementSearchAdapter = new TextGomElementSearchAdapter(_clientService, _searchConfiguration, _language, _formData, _isMetadataForm, gomElementName);
				searchResults.addAll(textGomElementSearchAdapter.process(_replace));

			// CMS_INPUT_DOM: Use a DomGomElementSearchAdapter for processing.
			} else if ("CMS_INPUT_DOM".equals(gomElementTag)) {
				final DomGomElementSearchAdapter domGomElementSearchAdapter = new DomGomElementSearchAdapter(_clientService, _searchConfiguration, _language, _formData, _isMetadataForm, gomElementName);
				searchResults.addAll(domGomElementSearchAdapter.process(_replace));

			// CMS_INPUT_DOMTABLE: Use a DomTableGomElementSearchAdapter for processing.
			} else if ("CMS_INPUT_DOMTABLE".equals(gomElementTag)) {
				final DomTableGomElementSearchAdapter domTableGomElementSearchAdapter = new DomTableGomElementSearchAdapter(_clientService, _searchConfiguration, _language, _formData, _isMetadataForm, gomElementName);
				searchResults.addAll(domTableGomElementSearchAdapter.process(_replace));

			// FS_LIST: Use an FsListGomElementSearchAdapter for processing.
			} else if ("FS_LIST".equals(gomElementTag)) {
				final FsListGomElementSearchAdapter fsListGomElementSearchAdapter = new FsListGomElementSearchAdapter(_clientService, _searchConfiguration, _language, _formData, _isMetadataForm, gomElementName);
				searchResults.addAll(fsListGomElementSearchAdapter.process(_replace));
			}
		}

		return searchResults;
	}
}