package de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.EditorValue;
import de.espirit.firstspirit.access.store.mediastore.File;
import de.espirit.firstspirit.access.store.mediastore.Media;
import de.espirit.firstspirit.access.store.mediastore.Picture;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.SearchAdapterUtil;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.formdata.FormDataSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.MediaSearchResult;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;


/**
 * Search adapter to perform search and replacement processes on elements of type {@link Media}.
 */
public class MediaStoreElementSearchAdapter extends StoreElementSearchAdapter {


	public MediaStoreElementSearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Media medium) {
		super(clientService, searchConfiguration, medium);
	}


	/**
	 * Process the store element, regarding replacement and single-stepping settings.
	 *
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	@Override
	protected List<SearchResult> process() {
		final List<SearchResult> searchResults = new ArrayList<SearchResult>();

		final Media medium = (Media) _element;

		// Process the medium's file name.
		final String mediumFilename = medium.getFilename();
		if (mediumFilename != null && !mediumFilename.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(mediumFilename, _searchConfiguration, _replace);
			if (result.getProcessingCount() > 0) {
				if (_replace && result.getReplacementStatus()) {
					medium.setFilename(result.getProcessedText());
				}
				searchResults.add(new MediaSearchResult(MediaSearchResult.Field.FILENAME, medium.getProject().getMasterLanguage(), result.getProcessingCount()));
			}
		}

		final List<Language> languages = new ArrayList<Language>();
		if (medium.isLanguageDependent()) {
			languages.addAll(_searchConfiguration.getLanguages());
		} else {
			languages.add(medium.getProject().getMasterLanguage());
		}
		for (final Language language : languages) {

			// Process the medium's description, differentiating between Picture and File media.
			if (medium.getType() == Media.PICTURE) {
				final Picture picture = medium.getPicture(language);
				if (picture != null) {
					final String mediumDescription = picture.getDescription();
					if (mediumDescription != null && !mediumDescription.isEmpty()) {
						final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(mediumDescription, _searchConfiguration, _replace);
						if (result.getProcessingCount() > 0) {
							if (_replace && result.getReplacementStatus()) {
								picture.setDescription(result.getProcessedText());
							}
							searchResults.add(new MediaSearchResult(MediaSearchResult.Field.DESCRIPTION, language, result.getProcessingCount()));
						}
					}
				}
			} else if (medium.getType() == Media.FILE) {
				final File file = medium.getFile(language);
				if (file != null) {
					final String mediaDescription = file.getDescription();
					if (mediaDescription != null && !mediaDescription.isEmpty()) {
						final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(mediaDescription, _searchConfiguration, _replace);
						if (result.getProcessingCount() > 0) {
							if (_replace && result.getReplacementStatus()) {
								file.setDescription(result.getProcessedText());
							}
							searchResults.add(new MediaSearchResult(MediaSearchResult.Field.DESCRIPTION, language, result.getProcessingCount()));
						}
					}
				}
			}

		}

		// If the element has metadata, the user may change metadata, and the metadata template is supported, process that data using a FormDataSearchAdapter.
		if (medium.hasMeta() && medium.getPermission().canMetaChange() && _clientService.isMetadataTemplateSupported()) {
			final FormData metaFormData = medium.getMetaFormData();
			if (metaFormData != null) {
				final FormDataSearchAdapter formDataSearchAdapter = new FormDataSearchAdapter(_clientService, _searchConfiguration, EditorValue.SOLE_LANGUAGE, metaFormData, true);
				searchResults.addAll(formDataSearchAdapter.process(_replace));
				if (_replace) {
					medium.setMetaFormData(metaFormData);
				}
			}
		}

		return searchResults;
	}
}