package de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement;

import de.espirit.firstspirit.access.project.Project;
import de.espirit.firstspirit.access.project.TemplateSet;
import de.espirit.firstspirit.access.store.templatestore.ChannelSourceProvider;
import de.espirit.firstspirit.access.store.templatestore.FormatTemplate;
import de.espirit.firstspirit.access.store.templatestore.GomSourceProvider;
import de.espirit.firstspirit.access.store.templatestore.RulesetContainer;
import de.espirit.firstspirit.access.store.templatestore.Snippet;
import de.espirit.firstspirit.access.store.templatestore.SnippetContainer;
import de.espirit.firstspirit.access.store.templatestore.Template;
import de.espirit.firstspirit.access.store.templatestore.TemplateStoreElement;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.SearchAdapterUtil;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.TemplateSearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * Search adapter to perform search and replacement processes on template elements (objects based on sub-classes of {@link TemplateStoreElement}.
 */
public class TemplateStoreElementSearchAdapter extends StoreElementSearchAdapter {

	public TemplateStoreElementSearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final TemplateStoreElement template) {
		super(clientService, searchConfiguration, template);
	}


	/**
	 * Process the store element, regarding replacement and single-stepping settings.
	 *
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	@Override
	protected List<SearchResult> process() {
		final List<SearchResult> searchResults = new ArrayList<SearchResult>();

		// Template elements of type Template support a comment field.
		if (_element instanceof Template) {
			final Template template = (Template) _element;
			searchResults.addAll(processComment(template));
		}

		// Template elements of type GomSourceProvider support GOM source (form) definitions.
		if (_element instanceof GomSourceProvider) {
			final GomSourceProvider gomSourceProvider = (GomSourceProvider) _element;
			searchResults.addAll(processGomSource(gomSourceProvider));
		}

		// Template elements of type RulesetContainer support ruleset definitions.
		if (_element instanceof RulesetContainer) {
			final RulesetContainer rulesetContainer = (RulesetContainer) _element;
			searchResults.addAll(processRuleset(rulesetContainer));
		}

		// Template elements of type SnippetContainer support snippet definitions for header, extract, and thumbnail.

		if (_element instanceof SnippetContainer) {
			final SnippetContainer snippetContainer = (SnippetContainer) _element;
			searchResults.addAll(processSnippet(snippetContainer));
		}

		// Template elements of type ChannelSourceProvider support output channel definitions (e.g. HTML, FOP->PDF, etc.).
		if (_element instanceof ChannelSourceProvider) {
			final ChannelSourceProvider channelSourceProvider = (ChannelSourceProvider) _element;
			searchResults.addAll(processChannelSources(channelSourceProvider));
		}

		// Template elements of type FormatTemplate support a tool-tip.
		if (_element instanceof FormatTemplate) {
			final FormatTemplate formatTemplate = (FormatTemplate) _element;
			searchResults.addAll(processTooltip(formatTemplate));
		}

		return searchResults;
	}


	/**
	 * Process a template's comment field and return an accordingly configured {@link SearchResult}.
	 *
	 * @param template The {@link Template} element to process.
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	private List<TemplateSearchResult> processComment(@NotNull final Template template) {
		final String templateComment = template.getComment();
		if (templateComment != null && !templateComment.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(templateComment, _searchConfiguration, _replace);
			if (result.getProcessingCount() > 0) {
				if (_replace && result.getReplacementStatus()) {
					template.setComment(result.getProcessedText());
				}
				return Collections.singletonList(new TemplateSearchResult(TemplateSearchResult.Tab.PROPERTIES, result.getProcessingCount()));
			}
		}
		return Collections.emptyList();
	}


	/**
	 * Process a template's GOM source and return an accordingly configured {@link SearchResult}.
	 *
	 * @param gomSourceProvider The {@link GomSourceProvider} element to process.
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	private List<TemplateSearchResult> processGomSource(@NotNull final GomSourceProvider gomSourceProvider) {
		final String gomSource = gomSourceProvider.getGomSource();
		if (gomSource != null && !gomSource.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(gomSource, _searchConfiguration, _replace);
			if (result.getProcessingCount() > 0) {
				if (_replace && result.getReplacementStatus()) {
					gomSourceProvider.setGomSource(result.getProcessedText());
				}
				return Collections.singletonList(new TemplateSearchResult(TemplateSearchResult.Tab.GOM_SOURCE, result.getProcessingCount()));
			}
		}
		return Collections.emptyList();
	}


	/**
	 * Process a template's ruleset definition and return an accordingly configured {@link SearchResult}.
	 *
	 * @param rulesetContainer The {@link RulesetContainer} element to process.
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	private List<TemplateSearchResult> processRuleset(@NotNull final RulesetContainer rulesetContainer) {
		final String ruleset = rulesetContainer.getRulesetDefinition();
		if (!ruleset.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(ruleset, _searchConfiguration, _replace);
			if (result.getProcessingCount() > 0) {
				if (_replace && result.getReplacementStatus()) {
					rulesetContainer.setRulesetDefinition(result.getProcessedText());
				}
				return Collections.singletonList(new TemplateSearchResult(TemplateSearchResult.Tab.RULESET_DEFINITION, result.getProcessingCount()));
			}
		}
		return Collections.emptyList();
	}


	/**
	 * Process a template's snippet definitions and return an accordingly configured {@link SearchResult}.
	 *
	 * @param snippetContainer The {@link SnippetContainer} element to process.
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	private List<TemplateSearchResult> processSnippet(@NotNull final SnippetContainer snippetContainer) {
		final Snippet snippet = snippetContainer.getSnippet();
		int count = 0;

		// Process the snippet's header definition.
		final String snippetHeader = snippet.getHeader();
		if (snippetHeader != null && !snippetHeader.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult headerResult = SearchAdapterUtil.processText(snippetHeader, _searchConfiguration, _replace);
			if (headerResult.getProcessingCount() > 0) {
				if (_replace && headerResult.getReplacementStatus()) {
					snippet.setHeader(headerResult.getProcessedText());
				}
				count += headerResult.getProcessingCount();
			}
		}

		// Process the snippet's extract definition.
		final String snippetExtract = snippet.getExtract();
		if (snippetExtract != null && !snippetExtract.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult extractResult = SearchAdapterUtil.processText(snippetExtract, _searchConfiguration, _replace);
			if (extractResult.getProcessingCount() > 0) {
				if (_replace && extractResult.getReplacementStatus()) {
					snippet.setExtract(extractResult.getProcessedText());
				}
				count += extractResult.getProcessingCount();
			}
		}

		// Process the snippet's thumbnail definition.
		final String snippetThumbnail = snippet.getThumbnail();
		if (snippetThumbnail != null && !snippetThumbnail.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult thumbnailResult = SearchAdapterUtil.processText(snippetThumbnail, _searchConfiguration, _replace);
			if (thumbnailResult.getProcessingCount() > 0) {
				if (_replace && thumbnailResult.getReplacementStatus()) {
					snippet.setThumbnail(thumbnailResult.getProcessedText());
				}
				count += thumbnailResult.getProcessingCount();
			}
		}

		// If any occurrences of the search string have been encountered in the process, produce a SearchResult and return it in a List object.
		if (count > 0) {
			snippetContainer.setSnippet(snippet);
			return Collections.singletonList(new TemplateSearchResult(TemplateSearchResult.Tab.SNIPPET, count));
		}
		return Collections.emptyList();
	}


	@NotNull
	private List<TemplateSearchResult> processChannelSources(@NotNull final ChannelSourceProvider channelSourceProvider) {
		final List<TemplateSearchResult> searchResults = new ArrayList<TemplateSearchResult>();

		// Obtain a list of template sets used in the current project. Each template set can have channel source code in the currently processed template.
		final Project project = channelSourceProvider.getProject();
		final List<TemplateSet> templateSets = project.getTemplateSets();

		// Process each channel source definition individually and return single SearchResult objects to enable switching between tabs using the NavigateToElementJavaClientExecutableReportItem.
		for (final TemplateSet templateSet : templateSets) {
			final String channelSource = channelSourceProvider.getChannelSource(templateSet);
			if (channelSource != null && !channelSource.isEmpty()) {
				final SearchAdapterUtil.ProcessingResult channelSourceResult = SearchAdapterUtil.processText(channelSource, _searchConfiguration, _replace);
				if (channelSourceResult.getProcessingCount() > 0) {
					if (_replace && channelSourceResult.getReplacementStatus()) {
						channelSourceProvider.setChannelSource(templateSet, channelSourceResult.getProcessedText());
					}
					searchResults.add(new TemplateSearchResult(templateSet, channelSourceResult.getProcessingCount()));
				}
			}
		}

		return searchResults;
	}


	@NotNull
	private List<TemplateSearchResult> processTooltip(@NotNull final FormatTemplate formatTemplate) {
		final String templateTooltip = formatTemplate.getStyleNode().getToolTip();
		if (templateTooltip != null && !templateTooltip.isEmpty()) {
			final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(templateTooltip, _searchConfiguration, _replace);
			if (result.getProcessingCount() > 0) {
				if (_replace && result.getReplacementStatus()) {
					formatTemplate.getStyleNode().setToolTip(result.getProcessedText());
				}
				return Collections.singletonList(new TemplateSearchResult(TemplateSearchResult.Tab.PROPERTIES, result.getProcessingCount()));
			}
		}
		return Collections.emptyList();
	}
}
